///////////////////////////////////////////////////////////////////////////////
// AW7HUD
// Heat vision stuff as well as other AW7-specific overlays
///////////////////////////////////////////////////////////////////////////////
class AW7HUD extends AWWrapHUD;

const TIME_X = 0.95;
const TIME_Y = 0.95;

///////////////////////////////////////////////////////////////////////////////
// Cross for drawing police radio beacons
///////////////////////////////////////////////////////////////////////////////
var() Texture TheCross;
var() Texture RadarStar;			// Starfish for grab bag radar

var color GreenColor;

///////////////////////////////////////////////////////////////////////////////
// PostRender
// Render heat vision in addition to normal HUD overlays.
///////////////////////////////////////////////////////////////////////////////
simulated event PostRender(canvas Canvas)
{
	local HUD H;
	local float YL,YPos;
	local Pawn P;

	// If a movie is playing, let Super handle everything
	if(TextureMovie != NONE && TextureMovie.Movie != NONE && TextureMovie.Movie.IsPlaying())
	{
		Super.PostRender(Canvas);
		return;
	}

	// If there's an active screen, check to see if it wants the hud
	// If there's a root window running then never show the hud
	if ((OurPlayer.CurrentScreen == None || OurPlayer.CurrentScreen.ShouldDrawHUD()) && !AreAnyRootWindowsRunning())
	{
		if ( !OurPlayer.bBehindView )
		{
			// Draw your foot in now
			if ( (PawnOwner != None) && (PawnOwner.MyFoot != None) )
				PawnOwner.MyFoot.RenderOverlays(Canvas);
		}

		// Do rest of weapons and hud
		if ( !OurPlayer.bBehindView )
		{
			P = Pawn(OurPlayer.ViewTarget);
			if ( (P != None) && (P.Weapon != None) )
				P.Weapon.RenderOverlays(Canvas);
		}

		// Of course, cast OurPlayer to your BulletPlayer class instead of ED_HeatVisionPlayer
		if (AW7DudePlayer(OurPlayer) != None && AW7DudePlayer(OurPlayer).bHeatVisionActive)
			AW7DudePlayer(OurPlayer).RenderHeatvisionOverlay(Canvas);

		DisplayMessages(Canvas);

		DrawTimeOnScreen(Canvas);

		bHideCenterMessages = DrawLevelAction(Canvas);

		if ( !bHideCenterMessages && (OurPlayer.ProgressTimeOut > Level.TimeSeconds) )
			DisplayProgressMessage(Canvas);

		if ( bBadConnectionAlert )
			DisplayBadConnectionAlert();

		if ( bShowDebugInfo )
		{
			YPos = 5;
			UseSmallFont(Canvas);
			OurPlayer.ViewTarget.DisplayDebug(Canvas,YL,YPos);
		}
		else for ( H=self; H!=None; H=H.NextHUD )
			H.DrawHUD(Canvas);

	// Give game info a chance to show debug stuff
	if (P2GameInfoSingle(Level.Game) != None)
		P2GameInfoSingle(Level.Game).RenderOverlays(Canvas);
	}

	// Display text saying who this version was issued to
	DrawIssuedToText(Canvas);

	if (OurPlayer.Pawn != None && OurPlayer.Pawn.FindInventoryType(class'CopRadioInv') != None)
		DrawRadioBeacons(Canvas);

	// Of course, cast OurPlayer to your BulletPlayer class instead of ED_HeatVisionPlayer
	if (AW7DudePlayer(OurPlayer) != None && AW7DudePlayer(OurPlayer).bWallHackActive)
		RenderWallHackOverlay(Canvas);

//	DrawCellBeacons(Canvas);
}

///////////////////////////////////////////////////////////////////////////////
// Draw wall hack overlay
///////////////////////////////////////////////////////////////////////////////
function RenderWallHackOverlay (Canvas Canvas)
{
	local Controller C;

	for (C=Level.ControllerList; C!=None; C=C.NextController)
	{
		if (C != Self && C.Pawn != None && !C.Pawn.bHidden)
		{
			if (!FastTrace(OurPlayer.Pawn.Location, C.Pawn.Location))
				Canvas.DrawActor(C.Pawn, true);
		}
	}
}


///////////////////////////////////////////////////////////////////////////////
// Draws a cop beacon on the radar.
///////////////////////////////////////////////////////////////////////////////
function DrawBeaconOnRadar (CopRadioBeacon Target, Canvas Canvas)
{
	local vector MyLocation;
	local vector TargetLocation;
	local vector DiffLocation;
	local vector X,Y,Z;

	local float  ScreenPosX;
	local float  ScreenPosY;

	local string DistanceInfo;
	local string HealthInfo;
	local string NameInfo;
	local color OurDrawColor;
	//local string WpnInfo, InvInfo;

	if (OurPlayer.ViewTarget != None)
	{
		MyLocation = OurPlayer.ViewTarget.Location;
		if (Pawn(OurPlayer.Viewtarget) != None)
			MyLocation.Z += Pawn(OurPlayer.Viewtarget).EyeHeight;
	}
	else
		MyLocation = OurPlayer.Location;

	TargetLocation = Target.Location;
	TargetLocation.Z += Target.CollisionHeight / 2;

	DiffLocation = TargetLocation - MyLocation;

	GetAxes(Normalize(OurPlayer.Rotation),X,Y,Z);

	if (DiffLocation Dot X > 0.70)
	{
		//log("Attempting to draw" @ Target @ "on" @ Canvas);
		ScreenPosX = (Canvas.ClipX / 2) + ( (DiffLocation Dot Y)) * ((Canvas.ClipX / 2) / Tan(OurPlayer.FovAngle * Pi/360)) / (DiffLocation Dot X);
		ScreenPosY = (Canvas.ClipY / 2) + (-(DiffLocation Dot Z)) * ((Canvas.ClipX / 2) / Tan(OurPlayer.FovAngle * Pi/360)) / (DiffLocation Dot X);

		Canvas.SetPos(ScreenPosX - 8, ScreenPosY - 8);
		OurDrawColor=RedColor;

		if (Target.TimerRate - Target.TimerCounter <= 10)
		{
			OurDrawColor.A = Int(255*(Target.TimerRate - Target.TimerCounter)/10);
			//log("it's"@OurDrawColor.A);
		}
		Canvas.DrawColor = OurDrawColor;
		Canvas.Style=ERenderStyle.STY_Alpha;
		Canvas.DrawIcon(TheCross, 0.5);

		DistanceInfo = String(Int(VSize(DiffLocation) / 48));

		Canvas.Font = Canvas.SmallFont;

		Canvas.SetPos(ScreenPosX + 10, ScreenPosY);
		Canvas.DrawText(DistanceInfo);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Draw all cop radio beacons
///////////////////////////////////////////////////////////////////////////////
function DrawRadioBeacons(canvas Canvas)
{
	local CopRadioBeacon B;

	foreach DynamicActors(class'CopRadioBeacon', B)
		DrawBeaconOnRadar(B, Canvas);
}

/*
function DrawCellBeacons(canvas Canvas)
{
	local CellBeacon B;

	foreach DynamicActors(class'CellBeacon', B)
		DrawBeaconOnRadar(B, Canvas);
}
*/

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function string LZero(int N, int Places)
{
	local string S;
	local int i;

	S = String(N);
	for (i=0; len(S)<Places; i++)
		S = "0" $ S;

	return S;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function DrawTimeOnScreen(Canvas C)
{
	local float Elapsed;
	local int Hours;
	local int Minutes;
	local float HrMin;
	local int Seconds;
	local string FinalOut;
	local float Excess;
	local int Centi;

	if (AW7GameState(P2GameInfoSingle(Level.Game).TheGameState) == None
		|| !AW7GameInfo(Level.Game).bDrawTime)
		return;

	Elapsed = AW7GameState(P2GameInfoSingle(Level.Game).TheGameState).TimeElapsed;

	Hours = int(Elapsed/3600);

	Minutes = Int((Elapsed - float(Hours*3600)) / 60);

	Seconds = Int((Elapsed - float(Hours*3600) - float(Minutes*60)));

	Excess = Hours*3600 + Minutes*60 + Seconds;
	Centi = int((Elapsed - Excess) * 100);

//	Minutes = int(Elapsed/60);

//	HrMin = Hours*3600 + Minutes*60;

//	Seconds = int(Elapsed - HrMin);
//	Excess = Elapsed - HrMin - Seconds;
//	Centi = int(Excess * 100);

	FinalOut = LZero(Hours,2) $ ":" $ LZero(Minutes, 2) $ ":" $ LZero(Seconds, 2) $ "." $ LZero(Centi, 2);

	MyFont.TextColor = WhiteColor;
	MyFont.DrawTextEX(C, CanvasWidth, TIME_X * CanvasWidth, TIME_Y * CanvasHeight, FinalOut, 1, true, EJ_Right);
	MyFont.TextColor = RedColor;
}

///////////////////////////////////////////////////////////////////////////////
// Draw radar showing other people around you
///////////////////////////////////////////////////////////////////////////////
simulated function DrawRadar(canvas Canvas, float Scale)
{
	local float UseSize, dist, radarx, radary, targetx, targety, RadarTimer;
	local float pheight, iconsize, glowalpha, sy, fishx, fishy;
	local Pickup radarp;
	local vector radarf;
	local int i;
	local bool bShowMouseHint, bIsMP;
	local vector dir;
	local GameReplicationInfo gri;

	Super.DrawRadar(Canvas, Scale);

	if(!OurPlayer.RadarTargetStats() && !OurPlayer.RadarTargetKilling() && OurPlayer.ShowRadarFull()
		&& AW7DudePlayer(OurPlayer).bDrawGoodies)
	{
		radarx=HUD_RADAR_X*CanvasWidth;// - UseSize/2 - Scale*(RadarPlayer.USize/8);
		radary=HUD_RADAR_Y*CanvasHeight;// - UseSize/2 - Scale*(RadarPlayer.VSize/8);

		// Draw all the pawns within radar
		for(i = 0; i<AW7DudePlayer(OurPlayer).GoodieArray.Length; i++)
		{
			// convert 3d world coords to around the player in the radar coords
			radarp = AW7DudePlayer(OurPlayer).GoodieArray[i];
			if(radarp != None)
			{
				dir = radarp.Location - PawnOwner.Location;
				pheight = dir.z;

				// If you're within the immediate height of the dude, be drawn
				// (or if it's MP, always show others)
				if(abs(pheight) < OurPlayer.RadarMaxZ
					|| OurPlayer.RadarInDoors==0
					|| bIsMP)
				{
					CalcRadarDists(bIsMP, dir, dist);

					// If you're within the appropriate radius from the player, be drawn
					if((bIsMP
							&& dist < MP_RADAR_RADIUS)
						|| dist < OurPlayer.RadarShowRadius)
					{

						RadarFindFishLoc(dist, Scale, pheight, bIsMP, dir, fishx, fishy, iconsize);
						Canvas.SetPos(radarx + fishx, radary + fishy);

						// Special colors based on weapon power
						if(P2WeaponPickup(radarp) != None)
						{
							// Extremely violent weapons show up red
							if(class<P2Weapon>(P2WeaponPickup(radarp).InventoryType).default.ViolenceRank >= 5)
								Canvas.DrawColor = RedColor;
							// Other weapons show up yellow
							else if(class<P2Weapon>(P2WeaponPickup(radarp).InventoryType).default.ViolenceRank > 0)
								Canvas.DrawColor = YellowColor;
							// Non-violent weapons are white
							else
								Canvas.DrawColor = WhiteColor;
						}
						else if (MoneyPickup(radarp) != None) // money shows up green
							Canvas.DrawColor = GreenColor;
						else if (P2PowerupPickup(radarp) != None // food and health shows up red
							&& (class<P2PowerupInv>(P2PowerupPickup(radarp).InventoryType).default.bEdible
								|| MedkitPickup(radarp) != None))
							Canvas.DrawColor = RedColor;
						else // default is a white fish
							Canvas.DrawColor = WhiteColor;

						Canvas.Style = ERenderStyle.STY_Alpha;

						// Draw the fish
						if (P2WeaponPickup(radarp) != None)
							Canvas.DrawIcon(RadarGun, iconsize);
						else
							Canvas.DrawIcon(RadarStar, iconsize);

						Canvas.Style = ERenderStyle.STY_Normal;
					}
				}
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////
defaultproperties
{
	TheCross=Texture'AW7Tex.Misc.MyCross'
	RocketHint2="Use the mouse to control the rocket."
	GreenColor=(G=255,A=255)
	RadarStar=Texture'MpHUD.Radar.StarFish'
}
