///////////////////////////////////////////////////////////////////////////////
// CopRadioInv
///////////////////////////////////////////////////////////////////////////////
class CopRadioInv extends OwnedInv;

///////////////////////////////////////////////////////////////////////////////
// Public Vars
///////////////////////////////////////////////////////////////////////////////
var() class<CopRadioBeacon> BeaconClass;
var() Array<Sound> RadioSounds;

///////////////////////////////////////////////////////////////////////////////
// Internal vars
///////////////////////////////////////////////////////////////////////////////
var FPSPawn MyPlayerPawn;

// for police AI use
var PoliceController MyPoliceController;
var FPSPawn CurrentAttacker;
var bool bCalledForBackup;

///////////////////////////////////////////////////////////////////////////////
// When you're sent to jail most weapons are taken. The Radar things aren't.
// Perhaps they want to do something now.
///////////////////////////////////////////////////////////////////////////////
function AfterItsTaken(P2Pawn CheckPawn)
{
	Destroy();
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function Activate()
{
	local P2Pawn P;
	local PoliceController C;
	local int i;
	local float Range;

	if (MyPlayerPawn == None)
		return;

	// Play a radio sound
	i = Rand(RadioSounds.Length);
	MyPlayerPawn.PlaySound(RadioSounds[i]);

	// Spawn a beacon here so the player can see it, if he has a radio
	spawn(BeaconClass,,,MyPlayerPawn.Location);

	// Search for people who care
	foreach DynamicActors(class'P2Pawn', P)
	{
		// don't allow pawns in slider stasis to respond
		if (!P.bSliderStasis)
		{
			Range = VSize(P.Location - MyPlayerPawn.Location);

			// Cops and cops only respond to the call... military and swat have no radio and don't really care.
			if (P.bAuthorityFigure && P.Controller.IsA('AW7InvPoliceController'))
			{
				AW7InvPoliceController(P.Controller).CopRadioAlert(MyPlayerPawn.Location, MyPlayerPawn);
/*
				C = PoliceController(P.Controller);
				// He's not dressed as a cop, and we don't already hate him
				if (!C.DudeDressedAsCop(MyPlayerPawn) && C.Attacker != MyPlayerPawn)
				{
					if(C.bStasis)
						C.ComeOutOfStasis(false);
					// If he's close and we can see him, arrest him
					if (Range <= 1000 && FastTrace(MyPlayerPawn.Location, P.Location))
					{
						// Arrest the guy
						C.SetAttacker(MyPlayerPawn);
						C.GotoState('HandleGuyWithBigGun');
					}
					// We don't know he's the dude, so let's help him
					//else if (C.InterestPawn == None && C.Attacker == None)
					else if (C.Attacker == None)
					{
						// Say something
						C.Say(C.MyPawn.myDialog.lCop_RadioBack);
						C.InterestPawn = MyPlayerPawn;
						C.SetEndPoint(MyPlayerPawn.Location, C.DEFAULT_END_RADIUS);
						//C.SetNextState('WatchGuyWithSmallGun');
						C.SetNextState('WatchForViolence');
						C.GotoStateSave('RunToInterest');
					}
				}
				// Help out your fellow "cop" in need of backup, assuming we're not busy with anything else right now.
				//else if (C.InterestPawn == None && C.Attacker == None)
				else if (C.Attacker == None)
				{
					if(C.bStasis)
						C.ComeOutOfStasis(false);
					// Say something
					if (Range <= 1000)
						C.Say(C.MyPawn.myDialog.lCop_GoingToInvestigate);
					else
						C.Say(C.MyPawn.myDialog.lCop_RadioBack);
					C.InterestPawn = MyPlayerPawn;
					C.SetEndPoint(MyPlayerPawn.Location, C.DEFAULT_END_RADIUS);
					C.SetNextState('WatchForViolence');
					C.GotoStateSave('RunToInterest');
				}*/
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function PoliceActivate()
{
	local P2Pawn P;
	local PoliceController C;
	local int i;
	local float Range;
	local P2Player P2P;
	local Inventory inv;

	if (MyPlayerPawn == None)
		return;

	// set flag so we don't use the radio over and over
	bCalledForBackup=true;

	// Play a radio sound
	i = Rand(RadioSounds.Length);
	MyPlayerPawn.PlaySound(RadioSounds[i]);

	// Spawn a beacon here so the player can see it, if he has a radio
	spawn(BeaconClass,,,MyPlayerPawn.Location);

	// If the dude has a radio, his should go off too
	P2P = P2GameInfo(Level.Game).GetPlayer();
	P = P2P.MyPawn;
	inv = P.FindInventoryType(class);
	if (Inv != None)
	{
		P.PlaySound(RadioSounds[i]);
		P2P.ClientMessage("Your radio is going off...");
	}

	// Search for people who care
	foreach DynamicActors(class'P2Pawn', P)
	{
		// don't allow pawns in slider stasis to respond
		if (!P.bSliderStasis)
		{
			Range = VSize(P.Location - MyPlayerPawn.Location);

			// Cops and cops only respond to the call... military and swat have no radio and don't really care.
			if (P.bAuthorityFigure && P.Controller.IsA('AW7InvPoliceController') && P != MyPlayerPawn)
			{
				AW7InvPoliceController(P.Controller).CopRadioAlert(MyPlayerPawn.Location, MyPlayerPawn);

/*				C = PoliceController(P.Controller);
				// Help out your fellow cop in need of backup, assuming we're not busy with anything else right now.
				//if (C.InterestPawn == None && C.Attacker == None)
				if (C.Attacker == None)
				{
					if(C.bStasis)
						C.ComeOutOfSliderStasis();
					// Say something
					if (Range <= 1000)
						C.Say(C.MyPawn.myDialog.lCop_GoingToInvestigate);
					else
					{
						// have their radio go off too
						MyPlayerPawn.PlaySound(RadioSounds[i]);
						C.Say(C.MyPawn.myDialog.lCop_RadioBack);
					}
					C.InterestPawn = MyPlayerPawn;
					C.SetEndPoint(MyPlayerPawn.Location, C.DEFAULT_END_RADIUS);
					C.SetNextState('WatchForViolence');
					C.GotoStateSave('RunToInterest');
				}*/
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Timer event for cops using radio.
///////////////////////////////////////////////////////////////////////////////
event Timer()
{
//	log(Name $ ":: Entered timer for"@MyPoliceController);
	// See if we need to call for backup
	if (CurrentAttacker != MyPoliceController.Attacker)
	{
		CurrentAttacker = MyPoliceController.Attacker;
//		log(Name $ ":: My cop's attacker is now" @ CurrentAttacker);
	}
	if (CurrentAttacker != None && MyPoliceController.bHateAttacker)
		if (!bCalledForBackup)
		{
//			log(Name $ ":: Calling for backup");
			PoliceActivate();
		}

	if (CurrentAttacker == None && bCalledForBackup)
	{
//		log(Name $ ":: No attacker, resetting backup call");
		bCalledForBackup = False;
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Idle state for police officers who get the radio
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
auto state Idle
{
	function BeginState()
	{
		log(Name $ ":: Cop Radio initialized");

		if (FPSPawn(Owner) == None)
		{
			GotoState('');
			return;
		}
		else
			MyPlayerPawn = FPSPawn(Owner);

		log(Name $ ":: My pawn is" @ MyPlayerPawn);

		// We only care if it's a police controller
		if (MyPlayerPawn.Controller != None && MyPlayerPawn.Controller.IsA('PoliceController'))
			MyPoliceController = PoliceController(MyPlayerPawn.Controller);
		else
		{
			GotoState('');
			return;
		}

		log(Name $ ":: My controller is" @ MyPoliceController);
		SetTimer(1.0, true);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
defaultproperties
{
	BeaconClass=class'CopRadioBeacon'
	PickupClass=class'CopRadioPickup'
	icon=Texture'AW7Tex.Icons.hud_copradio'

	RadioSounds[0]=Sound'WMaleDialog.wm_cop_radio1'
	RadioSounds[1]=Sound'WMaleDialog.wm_cop_radio2'
	RadioSounds[2]=Sound'WMaleDialog.wm_cop_radio3'
	RadioSounds[3]=Sound'WMaleDialog.wm_cop_radio4'

	Hint1="Press Enter to call for backup."
	Hint2="Only police officers can use this."
	MaxAmount=1
}
