///////////////////////////////////////////////////////////////////////////////
// HolyProjectile
// Holy hand grenade projectile.
///////////////////////////////////////////////////////////////////////////////
class HolyProjectile extends AWGrenadeProjectile;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
var() sound HallelujahSound;
var() class<P2WeaponPickup> PickupClass;
var bool bDoSplit;
var byte MaxChildren;

var bool bDoneAnthem;

///////////////////////////////////////////////////////////////////////////////
// Bounces off walls
///////////////////////////////////////////////////////////////////////////////
simulated function HitWall( vector HitNormal, actor Wall )
{
	local int i;
	local HolyProjectile gren;
	local vector NewVelocity;

	// Do bouncing first
	Super.HitWall(HitNormal, Wall);

	// If we're not a Bouncing Betty, quit
	if (!bDoSplit)
		return;

	for (i=0; i<MaxChildren; i++)
	{
		// Spawn Betty Daughters and send them flying in various directions.
		gren = spawn(class,Owner,,Location,rotator(HitNormal));
		NewVelocity.X = frand()*2*Velocity.X + 100;
		NewVelocity.Y = frand()*2*Velocity.Y + 100;
		NewVelocity.Z = frand()*2*Velocity.Z + 100;
		gren.SetupThrown(0);
		gren.Velocity = NewVelocity;
	}
	
	bDoSplit = False;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	if(DetonateTime > 0)
	{
		// Arm the grenade
		SetTimer(DetonateTime-2,false);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function MakeSmokeTrail()
{
	if(GTrail == None)
	{
		if(Level.Game.bIsSinglePlayer)
		{
			// Send player as owner so it will keep up in slomo time
			GTrail = spawn(class'HolyTrail',Owner,,Location);
			GTrail.SetBase(self);
		}
		else
			GTrail = spawn(class'HolyTrail',self,,Location);
	}
}

///////////////////////////////////////////////////////////////////////////////
// It explodes after DetonateTime no matter what
///////////////////////////////////////////////////////////////////////////////
simulated function Timer()
{
	if (bDoneAnthem)
		GenExplosion(Location, vect(0,0,1), None);
	else
	{
		PlaySound(HallelujahSound,SLOT_Misc,,,TransientSoundRadius,GetRandPitch());
		bDoneAnthem=true;
		SetTimer(2, false);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Blow up and generate effects
///////////////////////////////////////////////////////////////////////////////
simulated function GenExplosion(vector HitLocation, vector HitNormal, Actor Other)
{
	local GrenadeExplosion exp;
	local vector WallHitPoint;

	if(Role == ROLE_Authority)
	{
		if (!bDoneAnthem)
			PlaySound(HallelujahSound,SLOT_Misc,,,TransientSoundRadius,GetRandPitch());
		if(Other != None
			&& Other.bStatic)
		{
			// Make sure the force of this explosion is all the way against the wall that
			// we hit
			WallHitPoint = HitLocation - FORCE_RAD_CHECK*HitNormal;
			Trace(HitLocation, HitNormal, WallHitPoint, HitLocation);
		}
		else
			WallHitPoint = HitLocation;
		exp = spawn(class'HolyExplosion',GetMaker(),,HitLocation + ExploWallOut*HitNormal);
		exp.CheckForHitType(Other);
		exp.ShakeCamera(exp.ExplosionDamage);
		exp.ForceLocation = WallHitPoint;
	}
 	Destroy();
}

///////////////////////////////////////////////////////////////////////////////
// Have the un-armed projectile give the player back itself (as weapon ammo)
///////////////////////////////////////////////////////////////////////////////
simulated function MakePickup(Pawn Other)
{
	local Inventory Copy;
	local P2Weapon p2weap;
	local P2WeaponPickup gp;

	// Has to have bounced at least once to allow it to be picked up
	// (other-wise you'd be picking them up as you dropped them)
	if(bBouncedOnce
		&& Other != None)
	{
		if(Role == ROLE_Authority)
		{
			// Quickly make a pickup inside the player, then try to get the player
			// to touch it.. then remove them both, if he does.
			gp = spawn(PickupClass,,,Other.Location);
			if(gp != None)
			{
				gp.RespawnTime=0.0; // Don't allow this to respawn
				gp.AmmoGiveCount = 1;	// There's only one grenade here.
				gp.MPAmmoGiveCount = 1;
				gp.GotoState('Pickup');
				gp.Touch(Other);
				gp.Destroy();
				// Conditional destroy if we're a single player game--it gets better
				// service. A server/client game will always destroy the grenade.
				if(gp == None
					|| gp.bDeleteMe
					|| Level.NetMode == NM_DedicatedServer)
					Destroy();
			}
		}
		else
			Destroy();
	}
}

defaultproperties
{
	HallelujahSound=Sound'HabibDialog.habib_ailili'
	GrenadeBounce=Sound'WeaponSounds.rocket_bounce'
	MinSpeedForBounce=75.000000
	VelDampen=0.825000
	bDynamicLight=True
	StaticMesh=StaticMesh'AW7Mesh.Weapons.HolyGrenadeProj'
	AmbientGlow=255
	CollisionHeight=24.000000
	LightType=LT_Pulse
	LightBrightness=64.000000
	LightSaturation=255
	LightRadius=64.000000
	LightPeriod=32
	LightCone=128
	PickupClass=class'HolyWeapon'
	MaxChildren=4
}
