class HolyWeapon extends GrenadeWeapon;

///////////////////////////////////////////////////////////////////////////////
// Check to restore proper hints
///////////////////////////////////////////////////////////////////////////////
event TravelPostAccept()
{
	// don't use alt fire hints
	Super(P2Weapon).TravelPostAccept();
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function ThrowGrenade()
{
	local vector HitLocation, HitNormal, StartTrace, EndTrace, X,Y,Z;
	local actor HitActor;
	local HolyProjectile holy;

	if(P2AmmoInv(AmmoType) != None
		&& AmmoType.HasAmmo())
	{
		CalcAIChargeTime();
		FireOffset = AltFireOffset;

		// Alt-firing the grenade, drops it at your feet, and doesn't
		// arm it
		if(bAltFiring)
		{
			ChargeTime = 0.0;
			FireOffset = AltFireOffset;
		}
		else
			FireOffset = default.FireOffset;

		GetAxes(Instigator.GetViewRotation(),X,Y,Z);
		StartTrace = GetFireStart(X,Y,Z);
		AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);
		// Make sure we're not generating this on the other side of a thin wall
		// Also, bump anything along the way, so the grenade can break a window if
		// you're standing really close to one.
		//if(FastTrace(Instigator.Location, StartTrace))
		HitActor = Trace(HitLocation, HitNormal, Instigator.Location, StartTrace, true);
		if(HitActor == None
			|| (!HitActor.bStatic
				&& !HitActor.bWorldGeometry))
		{
			if(!bAltFiring)
				holy = spawn(class'HolyProjectile',Instigator,,StartTrace, AdjustedAim);
			else
				holy = spawn(class'HolyAltProjectile',Instigator,,StartTrace, AdjustedAim);

			// Make sure it got made, it could have gotten spawned in a wall and not made
			if(holy != None)
			{
				holy.Instigator = Instigator;

				// Compensate for catnip time, if necessary. Don't do this for NPCs
				if(FPSPawn(Instigator) != None
					&& FPSPawn(Instigator).bPlayer)
					ChargeTime /= Level.TimeDilation;

				holy.SetupThrown(ChargeTime);
				//gren.AddRelativeVelocity(Instigator.Velocity);
				P2AmmoInv(AmmoType).UseAmmoForShot();
				// Touch any actor that was in between, just in case.
				if(HitActor != None)
					HitActor.Bump(holy);

				// Holy Grenades become Bouncing Bettys in extended mode
				if (P2GameInfoSingle(Level.Game).VerifySeqTime() /* && !bAltFiring */ && Pawn(Owner).Controller.bIsPlayer)
					Holy.bDoSplit = True;
			}
		}
	}
	// Turn off the thing in his hand as it leaves
	if(ThirdPersonActor != None)
		ThirdPersonActor.bHidden=true;
}

defaultproperties
{
     bUsesAltFire=True
     HudHint1="Be careful! This thing packs"
     HudHint2="a punch!                    "
     AmmoName=Class'HolyAmmoInv'
     GroupOffset=10
     PickupClass=Class'HolyPickup'
     AttachmentClass=Class'HolyAttachment'
     ItemName="Jihad Grenade"
     Skins(0)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
     Skins(1)=Texture'AW7Tex.Weapons.PurpleGrenade'
}
