///////////////////////////////////////////////////////////////////////////////
// LauncherProjectile.
// Copyright 2002 Running With Scissors, Inc.  All Rights Reserved.
//
// This is the actual rocket that goes flying through the air.
// It fires straight, but it can run out fuel, at which point it tumbles
// to the ground, still potent.
// Detonates on contact with anything.
//
///////////////////////////////////////////////////////////////////////////////
class NukeProjectile extends LauncherProjectile;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Blow up and generate effects
///////////////////////////////////////////////////////////////////////////////
function GenExplosion(vector HitLocation, vector HitNormal, Actor Other)
{
	local RocketExplosion exp;
	local vector WallHitPoint, OrigLoc;
	local Actor ViewThing;
	
	if (Role < ROLE_Authority)
		return;

	if(Other != None
		&& Other.bStatic)
	{
		// Make sure the force of this explosion is all the way against the wall that
		// we hit
		OrigLoc = HitLocation;
		WallHitPoint = HitLocation - FORCE_RAD_CHECK*HitNormal;
		if(Trace(HitLocation, HitNormal, WallHitPoint, HitLocation) == None)
		{
			HitLocation = OrigLoc;
			WallHitPoint = OrigLoc;
		}
	}
	else
		WallHitPoint = HitLocation;

	exp = spawn(class'NukeExplosion',GetMaker(),,HitLocation + ExploWallOut*HitNormal);
	exp.CheckForHitType(Other);
	exp.ShakeCamera(exp.ExplosionDamage);
	exp.ForceLocation = WallHitPoint;

	// If we're controlling this particular rocket, tell the player about it
	if(Instigator != None
		&& P2Player(Instigator.Controller) != None
		&& P2Player(Instigator.Controller).ViewTarget == self)
	{
		// If we hit a person or animal or interesting thing, view it directly
		if(FPSPawn(Other) != None)
			ViewThing = Other;
		else // Otherwise, just watch the pretty explosion
			ViewThing = exp;
		P2Player(Instigator.Controller).RocketDetonated(ViewThing);
	}

 	Destroy();
}

state Flying
{
	simulated function BlastOff()
	{
		local vector Dir;
		local Actor useowner;

		Dir = vector(Rotation);
		// blast off
		Velocity += BlastOffVelMag*Dir;
		//Velocity.z/=8;
		Acceleration = ForwardAccelerationMag*Dir;

		if (Level.NetMode != NM_DedicatedServer)
		{
			if(Level.Game == None
				|| !Level.Game.bIsSinglePlayer)
				useowner = self;
			else // Send player as owner so it will keep up in slomo time
				useowner = Owner;

			// Send player as owner so it will keep up in slomo time
			if(STrail == None )
			{
				STrail = spawn(class'NukeTrail',useowner,,Location);
				if(Level.Game != None
					&& Level.Game.bIsSinglePlayer)
					STrail.SetBase(self);
			}
			if (FTrail == None )
			{
				FTrail = spawn(class'Fx.RocketFire',useowner,,Location);
				FTrail.SetBase(self);
			}
		}
	}

	simulated function BeginState()
	{
		local P2Player p2p;

		BlastOff();
		/*// Play flying sound
		Timer();*/
		// Turn on camera if necessary
		if(Instigator != None)
		{
			p2p = P2Player(Instigator.Controller);
			if(p2p != None
				&& p2p.bUseRocketCameras)
				// Make the rocket the view target
				p2p.StartViewingRocket(self);
		}
	}
}

defaultproperties
{
     RocketFlying=None
     RocketLaunch=Sound'AW7Sounds.nuke_launch'
     Skins(0)=Texture'AW7Tex.Weapons.NukeRocket'
}
