class SmegWeapon extends P2Weapon;

var int AltShotCountMaxForNotify;

simulated function PlayFiring()
{
	bForceReload=true;
	SetupMuzzleFlash();
	Super.PlayFiring();
}

simulated function PlayAltFiring()
{
	bForceReload=true;
	bAltFiring=True;
	IncrementFlashCount();
	// Play MP sounds on everyone's computers
	if(Level.Game == None
		|| !FPSGameInfo(Level.Game).bIsSinglePlayer)
		PlayOwnedSound(AltFireSound,SLOT_Interact,1.0,,,WeaponFirePitchStart + (FRand()*WeaponFirePitchRand),false);
	else // just on yours in SP games
		Instigator.PlaySound(AltFireSound, SLOT_None, 1.0, true, , WeaponFirePitchStart + (FRand()*WeaponFirePitchRand));

	PlayAnim('Shoot1', WeaponSpeedShoot2, 0.05);
}

///////////////////////////////////////////////////////////////////////////////
// This will randomly change the color and the size of the dynamic
// light associate with the flash. Change in each weapon's file,
// but call each time you start up the flash again.
// This function is also used by the third-person muzzle flash, so the
// colors will look the same
///////////////////////////////////////////////////////////////////////////////
simulated function PickLightValues()
{
	LightBrightness=150;
	LightSaturation=150;
	LightHue=12+FRand()*15;
	LightRadius=12+FRand()*6;
}

///////////////////////////////////////////////////////////////////////////////
// Make sure the muzzle flash renders, and move it around some
///////////////////////////////////////////////////////////////////////////////
simulated function SetupMuzzleFlash()
{
	bMuzzleFlash = true;
	bSetFlashTime = false;
	// Gets turned off in weapon, in RenderOverlays
	// Slightly change colors each time
	if(IsFirstPersonView()
		&& bDrawMuzzleFlash)
	{
		PickLightValues();
		bDynamicLight=bAllowDynamicLights;
	}
}


///////////////////////////////////////////////////////////////////////////////
// See what we hit
///////////////////////////////////////////////////////////////////////////////
function TraceFire( float Accuracy, float YOffset, float ZOffset )
{
	local int i;
	local PersonController perc;
	local vector dir;

	// Reduce the ammo only by 1 here, for the shotgun, but shoot
	// ShotCountMaxForNotify number of pellets each time.
	P2AmmoInv(AmmoType).UseAmmoForShot();

	perc = PersonController(Instigator.Controller);

	if (bAltFiring || (perc != None && perc.Target != None && perc.MyPawn.bAdvancedFiring))
	{
		for(i=0; i<AltShotCountMaxForNotify; i++)
		{
			Super.TraceFire(Accuracy, YOffset, ZOffset);
		}
	}
	else
	{
		for(i=0; i<ShotCountMaxForNotify; i++)
		{
			Super.TraceFire(Accuracy, YOffset, ZOffset);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Alt trace fire, plus check where to make the danger marker to 
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function TraceAltFire( float Accuracy, float YOffset, float ZOffset )
{
	TraceFire(Accuracy, YOffset, ZOffset);
}

state ClientFiring
{
}

state NormalFire
{
}

function Notify_ShootSMEG()
{
	if(bAltFiring)
		ShootSMEG();
}

function ShootSMEG()
{
}

simulated function Notify_RatchetSound()
{
	Instigator.PlayOwnedSound(ReloadSound, SLOT_Misc, 1.0);
}

///////////////////////////////////////////////////////////////////////////////
// play reloading sounds
///////////////////////////////////////////////////////////////////////////////
simulated function PlayReloading()
{
	// temp speed up because with no animation, you can't tell you're
	// why you can't shoot (becuase it's playing a reload anim)
	PlayAnim('Load', WeaponSpeedReload, 0.05);
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
	// Only primary fire for now
	bUsesAltFire=True
	//bUsesAltFire=False
	ViolenceRank=4
	RecognitionDist=1100.000000
	ShotCountMaxForNotify=16
	AltShotCountMaxForNotify=8
	AI_BurstCountMin=16
	holdstyle=WEAPONHOLDSTYLE_Double
	switchstyle=WEAPONHOLDSTYLE_Double
	firingstyle=WEAPONHOLDSTYLE_Double
	MinRange=200.000000
	ShakeOffsetTime=2.500000
	CombatRating=4.000000
	WeaponSpeedLoad=1.500000
	WeaponSpeedHolster=1.500000
	WeaponSpeedShoot1Rand=0.300000
	WeaponSpeedShoot2=0.250000
	WeaponSpeedShoot2Rand=0.300000
	WeaponSpeedIdle=0.800000
	AmmoName=Class'SmegAmmoInv'
	AutoSwitchPriority=3
	ShakeRotMag=(X=400.000000)
	ShakeRotTime=2.500000
	ShakeOffsetMag=(X=20.000000,Y=4.000000,Z=4.000000)
	TraceAccuracy=0.700000
	aimerror=400.000000
	AIRating=0.300000
	MaxRange=512.000000
	FireSound=Sound'WeaponSounds.shotgun_fire'
	AltFireSound=Sound'WeaponSounds.shotgun_fire'
	FlashOffsetY=-0.050000
	FlashOffsetX=0.060000
	FlashLength=0.050000
	MuzzleFlashSize=128.000000
	MFTexture=Texture'Timb.muzzle_flash.shotgun_corona'
	bDrawMuzzleFlash=True
	MuzzleFlashScale=2.400000
	MuzzleFlashStyle=STY_Normal
	InventoryGroup=3
	GroupOffset=10
	PickupClass=Class'SmegPickup'
	BobDamping=0.975000
	AttachmentClass=Class'SmegAttachment'
	ItemName="SMEG"
	Mesh=SkeletalMesh'AW7_Weapons.ED_SMEG_newest'
	Skins(0)=Texture'AW7Tex.Weapons.SMEG'
	Skins(1)=Texture'WeaponSkins.Dude_Hands'
	SoundRadius=255.000000
	WeaponsPackageStr="AW7_Weapons."
	ReloadSound=Sound'WeaponSounds.Sniper_EjectShell'
	//BettyIncSound=Sound'WeaponSounds.Explosion_Small'
}
