///////////////////////////////////////////////////////////////////////////////
// LauncherWeapon
// Copyright 2002 Running With Scissors, Inc.  All Rights Reserved.
//
// Rocket launcher
//
// Charged weapon. Needs to fuel each rocket. 
// If fired before it reaches the red in the fuel, it's a normal, forward
// thrusting rocket.
// If it reaches the red point and is fired, it shoots a seeking rocket,
// which targets the highest threat/ then nearest character.
// Alt-fire just is the same but doesn't shoot a seeking rocket at the end.
//
///////////////////////////////////////////////////////////////////////////////

class kCatLauncherWeapon extends LauncherWeapon;

///////////////////////////////////////////////////////////////////////////////
// When you're sent to jail most weapons are taken. The Radar things aren't. Perhaps
// they want to do something now.
///////////////////////////////////////////////////////////////////////////////
function AfterItsTaken(P2Pawn CheckPawn)
{
	Destroy();
}

function ServerFire()
{
	bAltFiring=false;
	ShootIt();
}

function ServerAltFire()
{
	bAltFiring=true;
	ShootIt();
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function Notify_ShootLauncher()
{
	local vector startloc, StartTrace, X,Y,Z, markerpos, HitNormal, HitLocation;
	local actor HitActor;
	local CatRocket2 lpro;
	local float giveback;
	
	if(AmmoType != None)
	{
		GetAxes(Instigator.GetViewRotation(),X,Y,Z);
		StartTrace = GetFireStart(X,Y,Z); 
		AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);
		CalcAIChargeTime();
		ShootStyleChanger=0;
		// Make sure we're not generating this on the other side of a thin wall
		//if(FastTrace(Instigator.Location, StartTrace))
		HitActor = Trace(HitLocation, HitNormal, Instigator.Location, StartTrace, true);
		StartLoc = sTARTTrace+ ((Instigator.CollisionRadius)*X);
		StartLoc.z+=(Instigator.CollisionRadius*0.5);
		if(HitActor == None
			|| (!HitActor.bStatic
				&& !HitActor.bWorldGeometry))
		{
			lpro = spawn(class'CatRocket2',Instigator,,StartLoc, Rotator(X));
			if(lpro != None)
			{
				lpro.Instigator = Instigator;
				// Compensate for catnip time, if necessary. Don't do this for NPCs
				if(FPSPawn(Instigator) != None
					&& FPSPawn(Instigator).bPlayer)
					ChargeTime /= Level.TimeDilation;
				//lpro.SetupShot(ChargeTime);
				lpro.AddRelativeVelocity(Instigator.Velocity);
				// Touch any actor that was in between, just in case.
				if(HitActor != None)
					HitActor.Bump(lpro);
				if(bAltFiring)
					lpro.bDoBounces=true;
			}
		}

		// Clear our target regardless
		CurrentTarget = None;
		// If we didn't make a valid rocket, then give him his ammo back
		if(lpro == None)
		{
			giveback = ChargeStartAmmo - AmmoType.AmmoAmount;
			if(giveback > 0)
				AmmoType.AddAmmo(giveback);
		}
		else
		{
			// Say we just fired
			ShotCount++;

			// Only make a new danger marker if the consecutive fires were as high
			// as the max
			if(ShotCount >= ShotCountMaxForNotify
				&& Instigator.Controller != None
				&& ShotMarkerMade != None)
			{
				// tell it we know this just happened, by recording it.
				ShotCount -= ShotCountMaxForNotify;
				
				// Records the first (gun fire)
				markerpos = Instigator.Location;
				
				// Primary (the gun shooting, making a loud noise)
				if(ShotMarkerMade != None)
				{
					ShotMarkerMade.static.NotifyControllersStatic(
						Level,
						ShotMarkerMade,
						FPSPawn(Instigator), 
						None, 
						ShotMarkerMade.default.CollisionRadius,
						markerpos);
				}
			}
		}
		// Reset the ammo you started with
		ChargeStartAmmo = AmmoType.AmmoAmount;
	}
}

state Active
{
	function BeginState()
	{
		Super.BeginState();
		P2Player(Instigator.Controller).ClientMessage("Cat Launcher selected");
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////
defaultproperties
	{
	GroupOffset=155
	ViolenceRank=1
	PickupClass=class'kCatLauncherPickup'
	AmmoName=class'kCatAmmoInv'
	FireSound=Sound'WeaponSounds.shotgun_catfire'
	ShotMarkerMade=class'FunnyThingMarker'
	FireOffset=(X=60.000000,Y=0.000000,Z=0.000000)
	Skins[1]=Texture'AnimalSkins.Cat_Orange'
	AttachmentClass=Class'kCatLauncherAttachment'
	}
