//=============================================================================
// MaDBombProjectile.
// Created on: 03/15/2005
// Written by Eduard Klemens
// Mail to: madjackal_x@hotmail.com.
//  2005, MaD Inc. All Rights Reserved.
//=============================================================================
class MaDBombProjectile extends AWGrenadeProjectile;

#exec OBJ LOAD FILE=MaDModsStatics.usx
#exec OBJ LOAD FILE=MaDModsSounds.uax

var MolotovWickFire	wickfire;		// Fire on the rope hanging out of bomb

///////////////////////////////////////////////////////////////////////////////
// Make the fire too
///////////////////////////////////////////////////////////////////////////////
simulated function PostBeginPlay()
{
	local Rotator uprot;

	Super.PostBeginPlay();

	// Call this on client or single player
	if ( Level.NetMode != NM_DedicatedServer)
	{
		wickfire = spawn(class'MolotovWickFire', self,,Location);
		wickfire.SetBase(self);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Send it with a certain speed and arm it
// A ChargeTime of 0 means it was alt-fired.
// The reason we don't just explicitly set some boolean like bArmed to say
// that the one 'thrown' is armed and the one 'set down' by alt-fire is
// not armed, is because, unlike a grenade, we say that if you alt-fire
// and gently set down a molotov that's lit but it falls a long way, it
// should still blow up.
///////////////////////////////////////////////////////////////////////////////
function SetupThrown(float ChargeTime)
{
	local Rotator uprot;

	// If it was tossed gently to the ground, it will arrive upright.
	if(ChargeTime == 0)
		StartSpinMag = 0;

	Super.SetupThrown(ChargeTime);

	if(ChargeTime == 0)
	{
		// Keep it upright
		uprot.Yaw=0;
		SetRotation(uprot);
	}
}

///////////////////////////////////////////////////////////////////////////////
// destroy fire
///////////////////////////////////////////////////////////////////////////////
simulated function Destroyed()
{
	if(wickfire != None)
	{
		wickfire.Destroy();
		wickfire = None;
	}
	Super.Destroyed();
}

///////////////////////////////////////////////////////////////////////////////
// Take damage or be force around
///////////////////////////////////////////////////////////////////////////////
function TakeDamage( int Dam, Pawn instigatedBy, Vector hitlocation,
							Vector momentum, class<DamageType> damageType)
{
	// Don't do anything if there's no damage
	if(Dam <= 0)
		return;
	// If you're moving, you can't take damage (unless it's
	// kicking/bludgeoning from anyone or bullet damage from an npc damage, then you can take it anytime)
	if(Physics == PHYS_Projectile
		&& !(ClassIsChildOf(damageType, class'BludgeonDamage')
			|| (ClassIsChildOf(damageType, class'BulletDamage')
				&& P2Player(Instigator.Controller) == None)))
		return;

	// several things don't hurt us
	if(ClassIsChildOf(damageType, class'AnthDamage')
		|| damageType == class'ElectricalDamage')
		return;

	// Make sure the one that blew it up gets the points for doing it.
	// Not for kicking, but still for scissors.
	if(!ClassIsChildOf(damageType, class'BludgeonDamage')
		|| damageType == class'CuttingDamage')
	{
		TransferInstigator(InstigatedBy);
	}

	SetPhysics(PHYS_Projectile);
	// Make sure it can move
	bBounce=true;
	Acceleration = default.Acceleration;

	// You can kick your own alt-fired grenades and kick normally thrown grenades, but
	// you can't kick another's (usually only in MP with other players) alt-fired grenades.
	// But this handles any and all damage from another other than the instigator. Basically
	// the alt-fired acts like a mine and as soon as they do anything to it, it blows up.
	if(MaDBombAltProjectile(self) != None
		&& !RelatedToMe(InstigatedBy))
	{
		Dam = Health;
	}
	else
	{
		// For kicking, shovel, baton type damages, the health is basically a
		// counter for how many hits (minus the last one) you can deliver
		// to a grenade before it blows up
		// Also, because of the nature of the instantaneous momentum delivery
		// and the full contact, don't add to the velocity, simply set the velocity.
		if(ClassIsChildOf(damageType, class'BludgeonDamage')
			&& damageType != class'CuttingDamage')
		{
			// If you kick it, you know 'own' it, so it won't blow up on you
			// anymore, but will blow up on anyone else (like, if you kick it
			// back at the attacker). The explosion will of course still hurt anyone.
			//log(self$" setting instigator "$Instigator$" to instigatedby "$Instigatedby);
			Instigator = instigatedBy;
			Dam = 1;
			Velocity=(Momentum/Mass);
		}
		else
		{
			Velocity+=(Momentum/Mass);
		}
		MakeSmokeTrail();
	}

	Health-=Dam;

	if(Health <= 0)
	{
		GenExplosion(HitLocation, vect(0, 0, 1), None);
		return;
	}

}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function BounceRecoil(vector HitNormal)
{
	local vector addvec;

//	log(self$" BounceRecoil");
	if(Physics != PHYS_Projectile)
		SetPhysics(PHYS_Projectile);
	// Make the normal a little crazy to promote some random bouncing
	addvec = (VRand()*0.05);
	if(addvec.z < 0)
		addvec.z = -addvec.z;
	HitNormal += addvec;

	Super.BounceRecoil(HitNormal);
}

///////////////////////////////////////////////////////////////////////////////
// Bounces off walls
///////////////////////////////////////////////////////////////////////////////
simulated function HitWall( vector HitNormal, actor Wall )
{
	local vector newhit, newnormal, EndPt;
	local bool bStopped;
	local float speed;
	local SmokeHitPuff smoke1;

	if(bBounce == true)
	{
		bBouncedOnce=true;

		speed = VSize(Velocity);
		// Check for a slowed speed
		if(speed < MinSpeedForBounce)
		{
			// Check for possible stop by seeing if we're stopped in z and
			// on the ground.
			EndPt = Location;
			EndPt.z-=DIST_CHECK_BELOW;
			// If there is a hit below (ground) then you are stopped.
			if(Trace(newhit, newnormal, EndPt, Location, false) != None)
				bStopped=true;
			else	// if we're not stopping, cap the speed at the minimum
			{
				Velocity = MinSpeedForBounce*Normal(Velocity);
				if(SameSpotBounce == 0)
				{
					SameSpot = Location;
					SameSpotBounce++;
				}
				else
				{
					if(VSize(SameSpot - Location) < SAME_SPOT_RADIUS)
					{
						SameSpotBounce++;
					}
					else
					{
						SameSpotBounce=0;
					}
					// We've bounce too many times in this spot--stop anyways.
					if(SameSpotBounce >= SAME_SPOT_BOUNCE_MAX)
						bStopped=true;
				}
			}
		}
		// If we've stopped, zero out the appropriate entries
		if(bStopped)
		{
			bBounce=false;
			Acceleration = vect(0, 0, 0);
			Velocity = vect(0, 0, 0);
			RotationRate.Pitch=0;
			RotationRate.Yaw=0;
			RotationRate.Roll=0;
			SetPhysics(PHYS_None);
			KillSmokeTrail();
		}
		else	// do bouncing
			BounceRecoil(HitNormal);

		// Throw out some hit effects, like dust or sparks
		smoke1 = spawn(class'Fx.SmokeHitPuffMelee',Owner,,Location,rotator(HitNormal));
		if(!bStopped)
			// play a noise
			smoke1.PlaySound(GrenadeBounce,,,,TransientSoundRadius,GetRandPitch());
	}
}

///////////////////////////////////////////////////////////////////////////////
// Have the un-armed projectile give the player back itself (as weapon ammo)
///////////////////////////////////////////////////////////////////////////////
simulated function MakePickup(Pawn Other)
{
	local Inventory Copy;
	local P2Weapon p2weap;
	local MaDBombPickup gp;

	// Has to have bounced at least once to allow it to be picked up
	// (other-wise you'd be picking them up as you dropped them)
	if(bBouncedOnce
		&& Other != None)
	{
		if(Role == ROLE_Authority)
		{
			// Quickly make a pickup inside the player, then try to get the player
			// to touch it.. then remove them both, if he does.
			gp = spawn(class'MaDBombPickup',,,Other.Location);
			if(gp != None)
			{
				gp.RespawnTime=0.0; // Don't allow this to respawn
				gp.AmmoGiveCount = 1;	// There's only one grenade here.
				gp.MPAmmoGiveCount = 1;
				gp.GotoState('Pickup');
				gp.Touch(Other);
				gp.Destroy();
				// Conditional destroy if we're a single player game--it gets better
				// service. A server/client game will always destroy the grenade.
				if(gp == None
					|| gp.bDeleteMe
					|| Level.NetMode == NM_DedicatedServer)
					Destroy();
			}
		}
		else
			Destroy();
	}
}

///////////////////////////////////////////////////////////////////////////////
// We can hurt ourselves with this if it's bounced once.
// Otherwise, it hurts everyone else all the same
///////////////////////////////////////////////////////////////////////////////
simulated function ProcessTouch(Actor Other, Vector HitLocation)
{
	local SmokeHitPuff smoke1;

	if (Pawn(Other) != None
		&& Pawn(Other).Health > 0)
	{
		// If it was un-armed,
		// and the special alt-grenade,
		// allow it to be picked back up
		if(!bArmed
			&& MaDBombAltProjectile(self) != None
			&& Pawn(Other).bCanPickupInventory
			// Only allow maker to pick it back up
			&& MadeMe(Pawn(Other)))
		{
			// SP players can pick them back up after alt-fired
			if(Level.Game != None
				&& Level.Game.bIsSinglePlayer)
			{
				MakePickup(Pawn(Other));
				if(bDeleteMe)
					return;
			}
			else // Let the guy that dropped them run over them in MP
				// it's better for team games and we wanted something consistent in MP.
				return;
		}
		// If not, check to detonate
		else if(!RelatedToMe(Pawn(Other)))
		{
			GenExplosion(HitLocation,Normal(HitLocation-Other.Location), Other);
		}
	}
	else
	{
		// Bounce off static things
		if(Other.bStatic)
		{
			// Throw out some hit effects, like dust or sparks
			smoke1 = spawn(class'Fx.SmokeHitPuffMelee',Owner,,Location,rotator(Normal(HitLocation-Other.Location)));
			// play a noise
			smoke1.PlaySound(GrenadeBounce,,,,TransientSoundRadius,GetRandPitch());
			BounceRecoil(-Normal(Velocity));
		}
		else
			Other.Bump(self);
	}
}

///////////////////////////////////////////////////////////////////////////////
// It explodes after DetonateTime no matter what
///////////////////////////////////////////////////////////////////////////////
simulated function Timer()
{
	GenExplosion(Location, vect(0,0,1), None);
}

///////////////////////////////////////////////////////////////////////////////
// Blow up and generate effects
///////////////////////////////////////////////////////////////////////////////
simulated function GenExplosion(vector HitLocation, vector HitNormal, Actor Other)
{
	local MaDBombExplosion exp;
	local MaDBombBoom expb;
	local MaDFireExplosion expf;
	local vector WallHitPoint;

	if(Role == ROLE_Authority)
	{
		if(Other != None
			&& Other.bStatic)
		{
			// Make sure the force of this explosion is all the way against the wall that
			// we hit
			WallHitPoint = HitLocation - FORCE_RAD_CHECK*HitNormal;
			Trace(HitLocation, HitNormal, WallHitPoint, HitLocation);
		}
		else
			WallHitPoint = HitLocation;
		expb = spawn(class'MaDBombBoom',GetMaker(),,HitLocation + ExploWallOut*HitNormal);
		exp = spawn(class'MaDBombExplosion',GetMaker(),,HitLocation + ExploWallOut*HitNormal);
		expf = spawn(class'MaDFireExplosion',GetMaker(),,HitLocation + ExploWallOut*HitNormal);
		exp.CheckForHitType(Other);
		expb.ShakeCamera(exp.ExplosionDamage);
		exp.ShakeCamera(exp.ExplosionDamage);
		expf.ShakeCamera(exp.ExplosionDamage);
		expb.ForceLocation = WallHitPoint;
		exp.ForceLocation = WallHitPoint;
		expf.ForceLocation = WallHitPoint;
	}
 	Destroy();
}

///////////////////////////////////////////////////////////////////////////////
// JustThrown
// Don't let fire damage effect a molotov just after it was thrown. Wait a split
// second. Before, when this wasn't here, if you threw a molotov when you were
// on fire, it burst and would never really get thrown.
///////////////////////////////////////////////////////////////////////////////
auto state JustThrown
{
	///////////////////////////////////////////////////////////////////////////////
	// Take damage or be force around
	///////////////////////////////////////////////////////////////////////////////
	function TakeDamage( int Dam, Pawn instigatedBy, Vector hitlocation,
								Vector momentum, class<DamageType> damageType)
	{
		// several things don't hurt us
		if(damageType == class'BurnedDamage')
			return;

		Global.TakeDamage(Dam, instigatedBy, hitlocation, momentum, damageType);
	}
Begin:
	Sleep(0.5);
	GotoState('');
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     GrenadeBounce=Sound'MaDModsSounds.MaDBombBounce'
     DetonateTime=6.000000
     Health=5
     MyDamageType=Class'AW7MaDInventory.MaDBombDamage'
     AmbientSound=Sound'MaDModsSounds.MaDBombAmbient'
     StaticMesh=StaticMesh'MaDModsStatics.MaDBombPickup'
     DrawScale=1.100000
     SoundRadius=200.000000
     SoundVolume=60
     CollisionRadius=15.000000
     CollisionHeight=15.000000
}
