//=============================================================================
// MaDBombWeapon.
// Created on: 03/15/2005
// Written by Eduard Klemens
// Mail to: madjackal_x@hotmail.com.
//  2005, MaD Inc. All Rights Reserved.
//=============================================================================
class MaDBombWeapon extends GrenadeWeapon;

#exec OBJ LOAD FILE=MaDModsAnims.ukx

///////////////////////////////////////////////////////////////////////////////
// Vars, structs, consts
///////////////////////////////////////////////////////////////////////////////
var MaDBomb MBomb;

simulated function PostNetBeginPlay()
{
    local PlayerController PC;
	local Pawn P;

    Super.PostNetBeginPlay();

    P  = Pawn(Owner);
	PC = PlayerController(P.Controller);

   	if (P.IsLocallyControlled() && PC!=None && (!PC.bBehindView) )
	{
        MBomb = Spawn(Class'AW7MaDInventory.MaDBomb');
        AttachToBone(MBomb, 'MaDB');
    }
   	else
	{
        if(Instigator != None)
	   {
		 if(Instigator.Mesh != None)

             MBomb.Destroyed();
             MBomb = None;
	   }
	}

}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function Destroyed()
{
   Super.Destroyed();

  if( MBomb != None )
  {
    DetachFromBone(MBomb);
    MBomb.Destroy();
    MBomb = None;
  }
}

///////////////////////////////////////////////////////////////////////////////
// Play firing animation/sound/etc
// Set here that we want to reload after each throw
///////////////////////////////////////////////////////////////////////////////
simulated function PlayFiring()
{
	Super.PlayFiring();
	bForceReload=true;
}
simulated function PlayAltFiring()
{
	Super.PlayAltFiring();
	bForceReload=true;
}

///////////////////////////////////////////////////////////////////////////////
// Normal projectile fire, plus check where to make the danger marker to
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function ProjectileFire()
{
	// STUB
}
///////////////////////////////////////////////////////////////////////////////
// Normal projectile fire, plus check where to make the danger marker to
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function ProjectileAltFire()
{
	// STUB
}

///////////////////////////////////////////////////////////////////////////////
// WE don't throw grenades
///////////////////////////////////////////////////////////////////////////////
function ThrowGrenade()
{
	// STUB
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function Notify_ThrowMolotov()
{
	local vector HitLocation, HitNormal, StartTrace, EndTrace, X,Y,Z;
	local actor HitActor;
	local MaDBombProjectile molot;

	if(P2AmmoInv(AmmoType) != None
		&& AmmoType.HasAmmo())
	{
		CalcAIChargeTime();
		// Alt-firing the grenade, drops it at your feet, and doesn't
		// arm it
		if(bAltFiring)
		{
			ChargeTime = 0.0;
			FireOffset = AltFireOffset;
		}
		else
		{
			FireOffset = default.FireOffset;
		}

		GetAxes(Instigator.GetViewRotation(),X,Y,Z);
		StartTrace = GetFireStart(X,Y,Z);
		AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);
		// Make sure we're not generating this on the other side of a thin wall
		//if(FastTrace(Instigator.Location, StartTrace))
		HitActor = Trace(HitLocation, HitNormal, Instigator.Location, StartTrace, true);
		if(HitActor == None
			|| (!HitActor.bStatic
				&& !HitActor.bWorldGeometry))
		{
			if(bAltFiring)
				molot = spawn(class'MaDBombAltProjectile',Instigator,,StartTrace, AdjustedAim);
			else
				molot = spawn(class'MaDBombProjectile',Instigator,,StartTrace, AdjustedAim);

			// Only use up the shot and perform the setup if we successfully spawned
			// a cocktail
			if(molot != None)
			{
				molot.Instigator = Instigator;

				// Compensate for catnip time, if necessary. Don't do this for NPCs
				if(FPSPawn(Instigator) != None
					&& FPSPawn(Instigator).bPlayer)
					ChargeTime /= Level.TimeDilation;

				molot.SetupThrown(ChargeTime);
				P2AmmoInv(AmmoType).UseAmmoForShot();
				// Touch any actor that was in between, just in case.
				if(HitActor != None)
					HitActor.Bump(molot);
			}
		}
	}
	// Turn off the thing in his hand as it leaves
	if(ThirdPersonActor != None)
		ThirdPersonActor.bHidden=true;
}

///////////////////////////////////////////////////////////////////////////////
// If you drop your weapon, and you were charging (probably when you were dying)
// then drop a molotov too.
///////////////////////////////////////////////////////////////////////////////
function DropFrom(vector StartLocation)
{
    MBomb.Destroyed();

	// If you were charging, throw out a live one now
	if(IsInState('BeforeCharging')
		|| IsInState('Charging')
		|| IsInState('ChargeWaitGrenade'))
	{
		ChargeTime = ChargeTimeMinAI;
		Notify_ThrowMolotov();
	}

	Super.DropFrom(StartLocation);
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     ChargeDistRatio=1600.000000
     ChargeTimeMinAI=0.700000
     WeaponSpeedCharge=0.500000
     AltHint1="Press Alt Fire (Right Mouse Button) to place"
     AltHint2="bombs. They explode after several seconds.  "
     ViolenceRank=2
     RecognitionDist=500.000000
     MinRange=512.000000
     FirstPersonMeshSuffix="Molotov"
     WeaponSpeedShoot1Rand=0.100000
     WeaponSpeedIdle=0.500000
     AmmoName=Class'AW7MaDInventory.MaDBombAmmoInv'
     AutoSwitchPriority=0
     TraceAccuracy=0.100000
     AIRating=0.530000
     FireSound=Sound'WeaponSounds.molotov_fire'
     SelectSound=Sound'MaDModsSounds.MaDBombSelect'
     GroupOffset=5
     PickupClass=Class'AW7MaDInventory.MaDBombPickup'
     AttachmentClass=Class'AW7MaDInventory.MaDBombAttachment'
     ItemName="M@D Bomb"
     Mesh=SkeletalMesh'MaDModsAnims.MaD_LS_Bomb'
     Skins(0)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
}
