//=============================================================================
// MaDSyringeWeapon.
// Created on: 03/25/2005
// Written by Eduard Klemens
// Mail to: madjackal_x@hotmail.com.
//  2005, MaD Inc. All Rights Reserved.
//=============================================================================
class MaDSyringeWeapon extends P2Weapon;

#exec OBJ LOAD FILE=MaDModsSounds.uax
#exec OBJ LOAD FILE=MaDModsAnims.ukx

///////////////////////////////////////////////////////////////////////////////
// Vars, structs, consts
///////////////////////////////////////////////////////////////////////////////

var float WeaponSpeedShoot1MP;	// Allow faster shooting in multiplayer.
var float WeaponSpeedShoot2MP;
var MaDSyringe MSy;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	// If it's a MP game, make them fire faster
	if(Level.Game == None
		|| !Level.Game.bIsSinglePlayer)
	{
		WeaponSpeedShoot1 = WeaponSpeedShoot1MP;
		WeaponSpeedShoot2 = WeaponSpeedShoot2MP;
	}
}

simulated function PostNetBeginPlay()
{
    local PlayerController PC;
	local Pawn P;

    Super.PostNetBeginPlay();

    P  = Pawn(Owner);
	PC = PlayerController(P.Controller);

   	if (P.IsLocallyControlled() && PC!=None && (!PC.bBehindView) )
	{
        MSy = Spawn(Class'AW7MaDInventory.MaDSyringe');
        AttachToBone(MSy, 'Syringe');
    }
   	else
	{
        if(Instigator != None)
	   {
		 if(Instigator.Mesh != None)

             MSy.Destroyed();
             MSy = None;
	   }
	}

}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function DropFrom(vector StartLocation)
{

   Super.DropFrom(StartLocation);


     MSy.Destroyed();

}

simulated function Destroyed()
{
   Super.Destroyed();

  if( MSy != None )
  {
    DetachFromBone(MSy);
    MSy.Destroy();
    MSy = None;
  }
}


///////////////////////////////////////////////////////////////////////////////
// Throws like a dart.
//
// Only spawn them here in MP because the anim notify can't keep up
// when it plays so fast.
///////////////////////////////////////////////////////////////////////////////
function ProjectileFire()
{
	if(Level.Game == None
		|| !Level.Game.bIsSinglePlayer)
//		|| (Instigator != None
//			&& PersonController(Instigator.Controller) != None))
	{
		SpawnScissors(false);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Play firing animation/sound/etc
// Set here that we want to reload after each throw
///////////////////////////////////////////////////////////////////////////////
simulated function PlayFiring()
{
	Super.PlayFiring();
	bForceReload=true;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function PlayReloading()
{
	PlayAnim('Load', WeaponSpeedReload, 0.05);

	P2MocapPawn(Instigator).PlayWeaponSwitch(self);
}

///////////////////////////////////////////////////////////////////////////////
// Called on client's side to make the gun fire
// Check here to throw out danger markers to let people know the gun has gone
// off.
///////////////////////////////////////////////////////////////////////////////
simulated function LocalFire()
{
	local P2Player P;

	bPointing = true;

	// Same as the one in P2Weapon, but we don't do the camera shake here
	// We make it shake when he throws

	if ( Affector != None )
		Affector.FireEffect();
	PlayFiring();
}

///////////////////////////////////////////////////////////////////////////////
// Tell ammo to make this scissors type
///////////////////////////////////////////////////////////////////////////////
function SpawnScissors(bool bMakeSpinner)
{
	local vector HitLocation, HitNormal, StartTrace, EndTrace, X,Y,Z;
	local actor Other;
	local MaDSyringeProjectile sic;
	local P2Player p2p;

	if(AmmoType != None
		&& AmmoType.HasAmmo())
	{
		GetAxes(Instigator.GetViewRotation(),X,Y,Z);
		StartTrace = GetFireStart(X,Y,Z);
		AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);
		if(bMakeSpinner)
		{
			TurnOffHint();
			sic = spawn(class'MaDSyringeAltProjectile',Instigator,,StartTrace, AdjustedAim);
		}
		else
			sic = spawn(class'MaDSyringeProjectile',Instigator,,StartTrace, AdjustedAim);
		// Make sure it got made
		if(sic != None)
			P2AmmoInv(AmmoType).UseAmmoForShot();

		// Shake the view when you throw it
		if ( Instigator != None)
		{
			p2p = P2Player(Instigator.Controller);
			if (p2p!=None)
			{
				p2p.ClientShakeView(ShakeRotMag, ShakeRotRate, ShakeRotTime,
							ShakeOffsetMag, ShakeOffsetRate, ShakeOffsetTime);
			}
		}
	}
	// Turn off the thing in his hand as it leaves
	if(ThirdPersonActor != None)
		ThirdPersonActor.bHidden=true;
}

///////////////////////////////////////////////////////////////////////////////
// Throw the scissors like a dart
//
// Because the animation is played much more slowly in SP allow the
// notify to throw the scissors. In MP spawn them at the start
// of the animation
///////////////////////////////////////////////////////////////////////////////
simulated function Notify_ThrowScissors()
	{
	if(Level.Game != None
		&& Level.Game.bIsSinglePlayer)
		{
			SpawnScissors(false);
		}
	}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Idle
//////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Idle
{
	///////////////////////////////////////////////////////////////////////////////
	// Make sure you're attachment is visible on idle start
	///////////////////////////////////////////////////////////////////////////////
	function BeginState()
	{
		Super.BeginState();
		if(ThirdPersonActor != None)
			ThirdPersonActor.bHidden=false;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
	WeaponSpeedShoot1=2.00
	bUsesAltFire=True
     WeaponSpeedShoot1MP=4.000000
     WeaponSpeedShoot2MP=5.000000
     ViolenceRank=1
     RecognitionDist=300.000000
     ShotCountMaxForNotify=0
     holdstyle=WEAPONHOLDSTYLE_Toss
     switchstyle=WEAPONHOLDSTYLE_Toss
     firingstyle=WEAPONHOLDSTYLE_Toss
     bThrownByFiring=True
     NoAmmoChangeState="EmptyDownWeapon"
     ShakeOffsetTime=0.000000
     bAllowHints=True
     bShowHints=True
     HudHint1="Alt Fire (Right Mouse Button) throws"
     HudHint2="ricocheting scissors."
     bBumpStartsFight=False
     CombatRating=1.800000
     FirstPersonMeshSuffix="Scissors"
     WeaponSpeedLoad=1.250000
     WeaponSpeedReload=5.000000
     WeaponSpeedHolster=1.500000
     WeaponSpeedShoot1Rand=0.050000
     WeaponSpeedShoot2Rand=0.050000
     AmmoName=Class'AW7MaDInventory.MaDSyringeAmmoInv'
     AutoSwitchPriority=0
     FireOffset=(X=15.000000,Y=5.000000,Z=-5.000000)
     ShakeRotMag=(X=200.000000,Y=30.000000,Z=30.000000)
     ShakeRotRate=(X=3000.000000,Y=5000.000000,Z=5000.000000)
     ShakeOffsetMag=(X=0.000000,Y=0.000000,Z=0.000000)
     ShakeOffsetRate=(X=0.000000,Y=0.000000,Z=0.000000)
     TraceAccuracy=0.100000
     aimerror=500.000000
     MaxRange=1024.000000
     FireSound=Sound'WeaponSounds.scissors_fire1'
     SelectSound=Sound'MaDModsSounds.MaDSyringeSelect'
     InventoryGroup=6
     GroupOffset=7
     PickupClass=Class'AW7MaDInventory.MaDSyringePickup'
     BobDamping=0.975000
     AttachmentClass=Class'AW7MaDInventory.MaDSyringeAttachment'
     ItemName="M@D Syringe"
     Mesh=SkeletalMesh'MaDModsAnims.MaD_LS_Syringe'
     Skins(0)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
}
