//=============================================================================
// MaDZyclonBWeapon.
// Created on: 05/04/2005
// Written by Eduard Klemens
// Mail to: madjackal_x@hotmail.com.
//  2005, MaD Inc. All Rights Reserved.
//=============================================================================
class MaDZyclonBWeapon extends MaDZyclonBBaseWeapon;

#exec OBJ LOAD FILE=MaDModsSounds.uax
#exec OBJ LOAD FILE=MaDModsAnims.ukx

var MaDZyclonBCan Can;

simulated function PostNetBeginPlay()
{
    local PlayerController PC;
	local Pawn P;

    Super.PostNetBeginPlay();

    P  = Pawn(Owner);
	PC = PlayerController(P.Controller);

   	if (P.IsLocallyControlled() && PC!=None && (!PC.bBehindView) )
	{
        Can = Spawn(Class'AW7MaDInventory.MaDZyclonBCan');
        AttachToBone(Can, 'ZCan');
    }
   	else
	{
        if(Instigator != None)
	   {
		 if(Instigator.Mesh != None)

             Can.Destroyed();
             Can = None;
	   }
	}

}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function DropFrom(vector StartLocation)
{

   Super.DropFrom(StartLocation);


     Can.Destroyed();

}

simulated function Destroyed()
{
   Super.Destroyed();

  if( Can != None )
  {
    DetachFromBone(Can);
    Can.Destroy();
    Can = None;
  }
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Play firing animation/sound/etc
// Set here that we want to reload after each throw
///////////////////////////////////////////////////////////////////////////////
simulated function PlayFiring()
{
	Super.PlayFiring();
	bForceReload=true;
}
simulated function PlayAltFiring()
{
	Super.PlayAltFiring();
	bForceReload=true;
}

///////////////////////////////////////////////////////////////////////////////
// Normal projectile fire, plus check where to make the danger marker to
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function ProjectileFire()
{
	// STUB
}
///////////////////////////////////////////////////////////////////////////////
// Alt projectile fire, plus check where to make the danger marker to
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function ProjectileAltFire()
{
	// STUB
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function PlayReloading()
{
	PlayAnim('Load', WeaponSpeedReload, 0.05);
}

///////////////////////////////////////////////////////////////////////////////
// Called on client's side to make the gun fire
// Check here to throw out danger markers to let people know the gun has gone
// off.
///////////////////////////////////////////////////////////////////////////////
simulated function LocalFire()
{
	local P2Player P;

	bPointing = true;

	// Same as the one in P2Weapon, but we don't do the camera shake here
	// We make it shake when he throws

	if ( Affector != None )
		Affector.FireEffect();
	PlayFiring();
	if(Role < ROLE_Authority)
		GotoState('ClientFiring');
}

///////////////////////////////////////////////////////////////////////////////
// Same as above.. we don't want the shake here
///////////////////////////////////////////////////////////////////////////////
simulated function LocalAltFire()
{
	local PlayerController P;

	bPointing = true;

	// Same as the one in P2Weapon, but we don't do the camera shake here
	// We make it shake when he throws

	if ( Affector != None )
		Affector.FireEffect();
	PlayAltFiring();
	if(Role < ROLE_Authority)
		GotoState('ClientFiring');
}

///////////////////////////////////////////////////////////////////////////////
// Actually throw the head
///////////////////////////////////////////////////////////////////////////////
function ThrowIt(float StartSpeed)
{
	local vector HitLocation, HitNormal, StartTrace, EndTrace, X,Y,Z;
	local actor HitActor;
	local MaDZyclonBProjectile cowh;
	local P2Player p2p;

	if(AmmoType != None
		&& AmmoType.HasAmmo())
	{
		GetAxes(Instigator.GetViewRotation(),X,Y,Z);
		StartTrace = GetFireStart(X,Y,Z);
		AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);
		// Make sure we're not generating this on the other side of a thin wall
		// Also, bump anything along the way, so the grenade can break a window if
		// you're standing really close to one.
		HitActor = Trace(HitLocation, HitNormal, Instigator.Location, StartTrace, true);
		if(HitActor == None
			|| (!HitActor.bStatic
				&& !HitActor.bWorldGeometry))
		{
			cowh = spawn(class'MaDZyclonBProjectile',Instigator,,StartTrace, AdjustedAim);
			if(cowh != None)
			{
				P2AmmoInv(AmmoType).UseAmmoForShot();
				cowh.Instigator = Instigator;
				cowh.CalcStartVelocity(StartSpeed);
				cowh.AddRelativeVelocity(Instigator.Velocity);

				// Shake the view when you throw it (primary fire only)
				if(!bAltFiring)
				{
					if ( Instigator != None)
					{
						p2p = P2Player(Instigator.Controller);
						if (p2p!=None)
						{
							p2p.ClientShakeView(ShakeRotMag, ShakeRotRate, ShakeRotTime,
										ShakeOffsetMag, ShakeOffsetRate, ShakeOffsetTime);
						}
					}
				}
				// Touch any actor that was in between, just in case.
				if(HitActor != None)
					HitActor.Bump(cowh);
			}
		}
	}
	// Turn off the thing in his hand as it leaves
	if(ThirdPersonActor != None)
		ThirdPersonActor.bHidden=true;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function Notify_ThrowCowhead()
{
	ThrowIt(1.0);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function Notify_AltThrowCowhead()
{
	ThrowIt(0.15);
	TurnOffHint();
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Idle
//////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Idle
{
	simulated function EndState()
	{
		Super.EndState();
	}
	///////////////////////////////////////////////////////////////////////////////
	// Make sure you're attachment is visible on idle start
	///////////////////////////////////////////////////////////////////////////////
	simulated function BeginState()
	{
		Super.BeginState();
		if(ThirdPersonActor != None)
			ThirdPersonActor.bHidden=false;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     bUsesAltFire=True
     ViolenceRank=1
     RecognitionDist=900.000000
     ShotCountMaxForNotify=0
     holdstyle=WEAPONHOLDSTYLE_Carry
     switchstyle=WEAPONHOLDSTYLE_Carry
     firingstyle=WEAPONHOLDSTYLE_Carry
     bThrownByFiring=True
     NoAmmoChangeState="EmptyDownWeapon"
     bNoHudReticle=True
     ShakeOffsetTime=2.500000
     bAllowHints=True
     bShowHints=True
     HudHint1="Stand still and press Secondary Fire(Right Mouse Button)   "
     HudHint2="to place it safely. Then shoot it at a distance to trigger."
     bBumpStartsFight=False
     bHideFoot=True
     FirstPersonMeshSuffix="Cowhead"
     WeaponSpeedShoot1Rand=0.500000
     AmmoName=Class'AW7MaDInventory.MaDZyclonBAmmoInv'
     AutoSwitchPriority=0
     FireOffset=(X=20.000000,Z=-10.000000)
     ShakeRotMag=(X=400.000000,Z=200.000000)
     ShakeRotTime=2.500000
     ShakeOffsetMag=(X=20.000000,Y=3.000000,Z=3.000000)
     TraceAccuracy=0.100000
     aimerror=500.000000
     MaxRange=1024.000000
     FireSound=Sound'WeaponSounds.cowhead_fire'
     SelectSound=Sound'MaDModsSounds.MaDBaseySelect'
     InventoryGroup=7
     GroupOffset=2
     PickupClass=Class'AW7MaDInventory.MaDZyclonBPickup'
     BobDamping=0.975000
     AttachmentClass=Class'AW7MaDInventory.MaDZyclonBAttachment'
     ItemName="M@D Zyclon B"
     Mesh=SkeletalMesh'MaDModsAnims.MaD_LS_ZyclonB'
     Skins(0)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
}
