///////////////////////////////////////////////////////////////////////////////
// AW7Bystanders
// Normal AW bystanders, but use our new civilian controller in case we make
// any changes that need to affect all civilians.
//
// 7/30/07 (Kamek)
// 		Added "random mugger" code, random bystanders get turned into muggers!
//			This should really throw people off!
///////////////////////////////////////////////////////////////////////////////
class AW7Bystanders extends AW7Person
	placeable;

///////////////////////////////////////////////////////////////////////////////
// Consts
///////////////////////////////////////////////////////////////////////////////
const MUGGER_FREQ = 0.001;		// Chance that we'll be a random mugger.
const PICKPOCKET_FREQ = 0.5;	// Chance of a random mugger being a pickpocket

///////////////////////////////////////////////////////////////////////////////
// Vars
///////////////////////////////////////////////////////////////////////////////
var() class<AIController> PickpocketControllerClass;
var() class<AIController> MuggerControllerClass;

///////////////////////////////////////////////////////////////////////////////
// AddDefaultInventory
// Chance of bystander becoming a random mugger.
///////////////////////////////////////////////////////////////////////////////
function AddDefaultInventory()
{
	Super.AddDefaultInventory();

	// A random idea I had while looking at the old Postal 2 design docs, why
	// not have random bystanders turn into muggers? Currently there are only
	// preset muggers, like the priest on main street. This way ANYBODY could
	// be a mugger, and it'll keep the player on their toes... assuming anyone
	// bothers to play this to begin with.

	// Do this ONLY for random bystanders, do not turn cops or cashiers or
	// such!
	if (FRand() <= MUGGER_FREQ && Controller.Class == class'AW7CivilianController')
	{
		log(self@"became a random mugger!");

		Controller.Destroy();
		Controller = None;

		while (Controller == None)
		{
	        if (FRand() < PICKPOCKET_FREQ)
	        	Controller = spawn(PickpocketControllerClass);
	        else
				Controller = spawn(MuggerControllerClass);
		}

		Controller.Possess(self);
		AIController(Controller).Skill += SkillModifier;
		CheckForAIScript();

		// Give the non-pickpocketers a gun if they don't have one already
		if (Controller.Class == MuggerControllerClass && !bHasViolentWeapon)
		{
			CreateInventoryByClass(class'PistolWeapon');
			bHasViolentWeapon = True;
		}
	}
}

defaultproperties
{
	ControllerClass=class'AW7CivilianController'
	PickpocketControllerClass=class'AW7PickpocketController'
	MuggerControllerClass=class'AW7MuggerController'
	ChameleonSkins(0)="ChameleonSkins.FB__094__Fem_SS_Shorts"
	ChameleonSkins(1)="ChameleonSkins.MW__022__Avg_M_SS_Pants"
	ChameleonSkins(2)="ChameleonSkins.FB__112__Fat_F_SS_Pants"
	ChameleonSkins(3)="ChameleonSkins.MB__015__Avg_M_Jacket_Pants"
	ChameleonSkins(4)="ChameleonSkins.FB__120__Fem_LS_Pants"
	ChameleonSkins(5)="ChameleonSkins.MB__026__Avg_M_SS_Pants"
	ChameleonSkins(6)="ChameleonSkins.FM__095__Fem_SS_Shorts"
	ChameleonSkins(7)="ChameleonSkins.MB__048__Avg_M_SS_Pants"
	ChameleonSkins(8)="ChameleonSkins.FM__118__Fem_LS_Pants"
	ChameleonSkins(9)="ChameleonSkins.MB__060__Avg_M_SS_Shorts"
	ChameleonSkins(10)="ChameleonSkins.FW__083__Fem_LS_Skirt"
	ChameleonSkins(11)="ChameleonSkins.MB__104__Fat_M_SS_Pants"
	ChameleonSkins(12)="ChameleonSkins.FW__084__Fem_LS_Skirt"
	ChameleonSkins(13)="ChameleonSkins.MM__018__Avg_M_Jacket_Pants"
	ChameleonSkins(14)="ChameleonSkins.FW__085__Fem_LS_Skirt"
	ChameleonSkins(15)="ChameleonSkins.MM__027__Avg_M_SS_Pants"
	ChameleonSkins(16)="ChameleonSkins.FW__092__Fem_LS_Skirt"
	ChameleonSkins(17)="ChameleonSkins.MM__102__Fat_M_SS_Pants"
	ChameleonSkins(18)="ChameleonSkins.FW__096__Fem_SS_Shorts"
	ChameleonSkins(19)="ChameleonSkins.MM__061__Avg_M_SS_Shorts"
	ChameleonSkins(20)="ChameleonSkins.FW__113__Fat_F_SS_Pants"
	ChameleonSkins(21)="ChameleonSkins.MM__105__Fat_M_SS_Pants"
	ChameleonSkins(22)="ChameleonSkins.FW__119__Fem_LS_Pants"
	ChameleonSkins(23)="ChameleonSkins.MM__107__Fat_M_SS_Pants"
	ChameleonSkins(24)="ChameleonSkins.FW__124__Fem_LS_Pants"
	ChameleonSkins(25)="ChameleonSkins.MW__016__Avg_M_Jacket_Pants"
	ChameleonSkins(26)="ChameleonSkins.MW__017__Avg_M_Jacket_Pants"
	ChameleonSkins(27)="ChameleonSkins.MW__019__Avg_M_Jacket_Pants"
	ChameleonSkins(28)="ChameleonSkins.MW__021__Avg_M_Jacket_Pants"
	ChameleonSkins(29)="ChameleonSkins.MW__024__Avg_M_SS_Pants"
	ChameleonSkins(30)="ChameleonSkins.MW__025__Avg_M_SS_Pants"
	ChameleonSkins(31)="ChameleonSkins.MW__028__Avg_M_SS_Pants"
	ChameleonSkins(32)="ChameleonSkins.MW__062__Avg_M_SS_Shorts"
	ChameleonSkins(33)="ChameleonSkins.MW__098__Fat_M_Jacket_Pants"
	ChameleonSkins(34)="ChameleonSkins.MW__106__Fat_M_SS_Pants"
	ChameleonSkins(35)="ChameleonSkins.MW__160__Avg_M_SS_Pants"
	ChameleonSkins(36)="ChameleonSkins.FM__082__Fem_LS_Skirt"
	ChameleonSkins(37)="AW7PawnSkins.Skins.MW__225__Avg_M_SS_Pants"
	ChameleonSkins(38)="AW7PawnSkins.PaletteSwap.FB__200__Fem_SS_Shorts"
	ChameleonSkins(39)="AW7PawnSkins.PaletteSwap.FB__201__Fat_F_SS_Pants"
	ChameleonSkins(40)="AW7PawnSkins.PaletteSwap.FB__202__Fem_LS_Pants"
	ChameleonSkins(41)="AW7PawnSkins.PaletteSwap.FM__203__Fem_LS_Pants"
	ChameleonSkins(42)="AW7PawnSkins.PaletteSwap.FW__204__Fem_LS_Skirt"
	ChameleonSkins(43)="AW7PawnSkins.PaletteSwap.FW__205__Fem_LS_Skirt"
	ChameleonSkins(44)="AW7PawnSkins.PaletteSwap.FW__206__Fem_LS_Skirt"
	ChameleonSkins(45)="AW7PawnSkins.PaletteSwap.FW__207__Fat_F_SS_Pants"
	ChameleonSkins(46)="AW7PawnSkins.PaletteSwap.FW__208__Fem_LS_Pants"
	ChameleonSkins(47)="AW7PawnSkins.PaletteSwap.FW__209__Fem_LS_Pants"
	ChameleonSkins(48)="AW7PawnSkins.PaletteSwap.MM__210__Avg_M_SS_Pants"
	ChameleonSkins(49)="AW7PawnSkins.PaletteSwap.MM__211__Avg_M_SS_Shorts"
	ChameleonSkins(50)="AW7PawnSkins.PaletteSwap.MW__212__Avg_M_Jacket_Pants"
	ChameleonSkins(51)="AW7PawnSkins.PaletteSwap.MW__213__Avg_M_SS_Pants"
	ChameleonSkins(52)="AW7PawnSkins.PaletteSwap.MW__214__Avg_M_SS_Shorts"
	ChameleonSkins(53)="AW7PawnSkins.PaletteSwap.MW__215__Fat_M_SS_Pants"
	ChameleonSkins(54)="AW7PawnSkins.PaletteSwap.MW__216__Avg_M_SS_Pants"
//	ChameleonSkins(55)="AW7PawnSkins.fromhell.MB__300__Avg_M_SS_Pants"
//	ChameleonSkins(56)="AW7PawnSkins.fromhell.MM__301__Avg_M_SS_Pants"
//	ChameleonSkins(57)="AW7PawnSkins.fromhell.MW__302__Avg_M_SS_Shorts"
	ChameleonSkins(55)="AW7PawnSkins.Skins.FW__402__Fem_LS_LongSkirt"
	ChameleonSkins(56)="AW7PawnSkins.Skins.FW__403__Fem_LS_LongSkirt"
	ChameleonSkins(57)="AW7PawnSkins.Skins.FM__404__Fem_LS_LongSkirt"
	ChameleonSkins(58)="AW7PawnSkins.Skins.FW__405__Fem_LS_LongSkirt"
	ChameleonSkins(59)="AW7PawnSkins.Skins.FM__400__Fem_LS_LongSkirt"
//	ChameleonSkins(60)="AW7PawnSkins.Skins.MW__450__Avg_M_SS_Pants"
//	ChameleonSkins(64)="AW7PawnSkins.fromhell.FW__304__Fem_LS_Pants"
//	ChameleonSkins(65)="AW7PawnSkins.fromhell.MM__306__Avg_M_SS_Pants"
//	ChameleonSkins(66)="AW7PawnSkins.fromhell.MM__307__Avg_M_SS_Shorts"
//	ChameleonSkins(67)="AW7PawnSkins.fromhell.MW__308__Avg_M_SS_Shorts"
//	ChameleonSkins(68)="AW7PawnSkins.fromhell.MW__309__Avg_M_SS_Pants"
//	ChameleonSkins(61)="AW7PawnSkins.Skins.MW__451__Avg_M_SS_Shorts"
	ChameleonSkins(60)="AW7PawnSkins.Skins.FW__217__Fem_LS_Skirt"
	ChameleonSkins(61)="AW7PawnSkins.Skins.FW__218__Fem_SS_Shorts"
	ChameleonSkins(62)="AW7PawnSkins.Skins.FW__219__Fem_SS_Shorts"
	ChameleonSkins(63)="AW7PawnSkins.Skins.FW__220__Fem_LS_Skirt"
	ChameleonSkins(64)="AW7PawnSkins.Skins.MM__226__Avg_M_SS_Pants"
	ChameleonSkins(65)="End"
	// Non-retail body skins
//	ChameleonSkins(65)="AW7PawnSkins.Skins.FW__304__Fem_LS_Pants"
//	ChameleonSkins(66)="AW7PawnSkins.Skins.MB__300__Avg_M_SS_Pants"
//    ChameleonSkins(67)="AW7PawnSkins.Skins.MM__301__Avg_M_SS_Pants"
//    ChameleonSkins(68)="AW7PawnSkins.Skins.MM__306__Avg_M_SS_Pants"
//    ChameleonSkins(69)="AW7PawnSkins.Skins.MM__307__Avg_M_SS_Shorts"
//    ChameleonSkins(70)="AW7PawnSkins.Skins.MW__302__Avg_M_SS_Shorts"
//    ChameleonSkins(71)="AW7PawnSkins.Skins.MW__308__Avg_M_SS_Shorts"
//    ChameleonSkins(72)="AW7PawnSkins.Skins.MW__309__Avg_M_SS_Pants"
//    ChameleonSkins(73)="AW7PawnSkins.Skins.MW__451__Avg_M_SS_Shorts"
//	ChameleonSkins(74)="End"
	ChameleonMeshPkgs(0)="Characters"
	ChameleonMeshPkgs(1)="AW7Characters"
	bInnocent=True
	bUsePawnSlider=True
	Mesh=SkeletalMesh'Characters.Fem_SS_Shorts'
	Skins(0)=Texture'ChameleonSkins.BystandersF.XX__142__Fem_SS_Shorts'
}
