///////////////////////////////////////////////////////////////////////////////
// PickPockController
// Copyright 2002 RWS, Inc.  All Rights Reserved.
//
// High-level RWS AI Controllers for pick pockets
//
// Make a few (4 or so) homenodes in a level with the same
// tag so they can be the HomeTag of the pick pocket. They will
// pick the farthest one away from their current position to run
// to when they are in trouble.
//
// Walks towards prey, and when they bump into them, they take
// your money. Then causes dude to yell after a while, same with
// NPC's, they either fight or run screaming
//
// If triggered, they will walk to pick pocket the player specifically
//
// Similar to mugger controller from original p2
//
///////////////////////////////////////////////////////////////////////////////
class AW7PickpocketController extends AW7CivilianController;

///////////////////////////////////////////////////////////////////////////////
// Vars
///////////////////////////////////////////////////////////////////////////////
// User set vars

// Internal vars
var float TimeWatchingPrey;		// Vague time you've spent watching your prey. Eventually
								// used to determine to mug them or not

///////////////////////////////////////////////////////////////////////////////
// Const
///////////////////////////////////////////////////////////////////////////////
const FIND_VICTIM_RAD		= 3000;
const STARE_POSSIBLE_COUNT  = 2;
const STARE_COUNT			= 2;
const START_APPROACH_RADIUS	= 2500;
const RUN_TO_PREY			= 1800;
const WALK_TO_PREY			= 500;

const PICK_NPC_FOR_MUGGING		= 0.3;
const PICK_PLAYER_FOR_MUGGING	= 0.8;

const MUG_MAX_RADIUS		= 350;
const MUG_MIN_RADIUS		= 150;

const LOOK_SHIFTILY_FREQ	= 4;

const PREY_CHECK_TIME		= 1.0;
const WATCH_PREY_MIN		= 4.0;
const WATCH_PREY_MAX		= 60.0;
const MONEY_WAIT_UPDATE		= 0.7;

const PLAYER_WAIT_RAND		= 10;
const PLAYER_WAIT_BASE		= 10;
const PLAYER_WAIT_MIN		= 6;

const CASH_GIVE_RAND		= 50;
const CASH_GIVE_BASE		= 10;

const MUG_FREQ				= 3;


///////////////////////////////////////////////////////////////////////////////
// start: set up functions (from spawner and the like) using PawnInitialState
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Find a player and kick his butt
///////////////////////////////////////////////////////////////////////////////
function SetToPickPocketPlayer(FPSPawn PlayerP)
{
	local FPSPawn keepp;

	if(PlayerP == None)
		keepp = GetRandomPlayer().MyPawn;
	else
		keepp = PlayerP;

	// check for some one to attack
	if(keepp != None)
	{
		InterestPawn = keepp;
		if(PreyIsInvalid())
		{
			SetNextState('Thinking', 'TryAgain');
		}
		else
		{
			SetEndGoal(InterestPawn, MUG_MIN_RADIUS);
			ScreamState=SCREAM_STATE_NONE; // clear scream state before we run
			SetNextState('RunToTarget');
		}
	}
}
///////////////////////////////////////////////////////////////////////////////
// end: set up functions (from spawner and the like) using PawnInitialState
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Force them to mug the player
///////////////////////////////////////////////////////////////////////////////
function Trigger( actor Other, Pawn EventInstigator )
{
	local byte StateChange;

	MyPawn.bIgnoresSenses=false;
	MyPawn.bIgnoresHearing=false;

	if(!MyPawn.bPlayerIsFriend)
	{
		// We don't need a weapon to pickpocket
		//if(MyPawn.bHasViolentWeapon)
		//{
			SetToPickPocketPlayer(FPSPawn(Other));
			GotoNextState();
			if(InterestPawn != None)
			{
				if(!InterestPawn.bPlayer)
					SetNextState('DoNonPlayerMugging');
				else
					SetNextState('DoPlayerMugging');
			}
			StateChange = 1;
		//}
	}
	if(StateChange == 0)
		Super.Trigger(Other, EventInstigator);
}

///////////////////////////////////////////////////////////////////////////////
// Called by the interest points to see if we're interested in doing this
// Don't allow if we're focussed on someone
///////////////////////////////////////////////////////////////////////////////
function PerformInterestAction(InterestPoint IPoint, out byte AllowFlow)
{
	if(InterestPawn == None)
		AllowFlow=1;
}

///////////////////////////////////////////////////////////////////////////////
// Make me a marker
///////////////////////////////////////////////////////////////////////////////
function MakeMarker(class<TimedMarker> ADanger,
					FPSPawn originpawn, 
					FPSPawn creatorpawn)
{
	ADanger.static.NotifyControllersStatic(
		Level,
		ADanger,
		originpawn, 
		creatorpawn, 
		ADanger.default.CollisionRadius,
		originpawn.Location);
}

///////////////////////////////////////////////////////////////////////////
// Returns true if our interestpawn is dead or doing something else.
///////////////////////////////////////////////////////////////////////////
function bool PreyIsInvalid()
{
	return (InterestPawn == None
			|| InterestPawn.bDeleteMe
			|| InterestPawn.Health <= 0
			|| (AWBystanderController(InterestPawn.Controller) != None
				&& AWBystanderController(InterestPawn.Controller).Attacker != None));
}

///////////////////////////////////////////////////////////////////////////////
// You're getting the money
///////////////////////////////////////////////////////////////////////////////
function bool GrabbingMoney()
{
	local P2PowerupPickup ppick;

	// Check if it's still there
	if(Pickup(Focus) != None
		&& !Focus.bDeleteMe)
	{
		ppick = P2PowerupPickup(Focus);

		// Say you can grab it.
		MyPawn.bCanPickupInventory=true;
		// Force a touch to register the grab
		Focus.Touch(MyPawn);
		// Say you can't grab anything anymore
		MyPawn.bCanPickupInventory=false;
		return true;
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////
// Scope the people around you first and pick some hapless person
///////////////////////////////////////////////////////////////////////////
function PickSomePrey()
{
	local P2Pawn CheckP;
	local AWBystanderController perc;
	local bool bPickThisGuy;

	InterestPawn = None;
	// Make sure we are being rendered currently before starting our
	// mugging stuff
//	if(MyPawn.bHasViolentWeapon)
//		&& MyPawn.LastRenderTime + 1.0 >= Level.TimeSeconds)
//	{
		ForEach CollidingActors(class'P2Pawn', CheckP, FIND_VICTIM_RAD, MyPawn.Location)
		{
			// If not me
			if(CheckP != MyPawn
				// Not who we already looked at
				&& InterestPawn != CheckP
				// if still alive (and not dying)
				&& CheckP.Health > 0
				// Not persistant (those are important people)
				&& !CheckP.bPersistent
				// See if we have a line of sight to them
				&& CanSeePawn(MyPawn, CheckP)
				// Only try to prey on bystanders--dude is a bystander--and not cops 
				// or swat.
				&& CheckP.IsA('Bystander')
				// Not friends with me
				&& !SameGang(CheckP))
			{
				if(!CheckP.bPlayer)
				{
					// Make sure the person we're checking out isn't important
					// and isn't currently in trouble somehow or another
					perc = AWBystanderController(CheckP.Controller);
					if(perc != None
						&& perc.Attacker == None
						&& perc.InterestPawn == None
						//&& CashierController(perc) == None
						//&& MuggerController(perc) == None
						&& PickPockController(perc) == None)
					{
						bPickThisGuy=true;
					}
				}
				else	// Make sure the player has some money first
				{
					if(AWPlayer(CheckP.Controller) != None
						&& AWPlayer(CheckP.Controller).CashPlayerHas() > 0)
						bPickThisGuy=true;
				}

				if(bPickThisGuy)
				{
					// set our prey
					InterestPawn = CheckP;
					statecount = 1 + Rand(STARE_POSSIBLE_COUNT);
					return;
				}
			}
		}
//	}
}

///////////////////////////////////////////////////////////////////////////
// Setup to pickpocket them, bail if the target is no good any more
///////////////////////////////////////////////////////////////////////////
function PrepToMug()
{
	if(PreyIsInvalid())
	{
		GotoStateSave('Thinking');
	}
	else
	{
		if(!InterestPawn.bPlayer)
			GotoStateSave('DoNonPlayerMugging');
		else
			GotoStateSave('DoPlayerMugging');
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Decide what to do next
// We don't randomly chat with people
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Thinking
{
	ignores CanBeMugged;

Begin:
	// Tell others if you have a weapon out
	ReportViolentWeaponNoStasis();

	// Check first to try to mug someone
	PickSomePrey();

	if(InterestPawn != None)
		GotoStateSave('CheckOutPossiblePrey');

TryAgain:
	Sleep(FRand() + 0.5);

	// Check to do a patrol
	if(MyPawn.PatrolNodes.Length > 0)
	{
		Sleep(2.0);
		SetToPatrolPath();
		GotoNextState();
	}

	// walk to some random place I can see (not through walls)
	SetNextState('Thinking');
	if(!PickRandomDest())
		Goto('TryAgain');	// Didn't find a valid point, try again
	GotoStateSave('WalkToTarget');
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Look at someone and think about robbing them
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state CheckOutPossiblePrey
{
	ignores RespondToTalker, PerformInterestAction, RatOutAttacker, CheckDeadBody, CheckDeadHead,
		WatchFunnyThing, CheckDesiredThing, CheckForIntruder, CanStartConversation, 
		FreeToSeekPlayer, CanBeMugged, ForceGetDown;
	
	///////////////////////////////////////////////////////////////////////////////
	// If you lose sight of them, or they die, stop watching them
	///////////////////////////////////////////////////////////////////////////////
	function CheckPrey()
	{
		if(InterestPawn == None
			|| InterestPawn.Health <= 0
			|| !FastTrace(InterestPawn.Location, MyPawn.Location))
			WatchSomeoneElse();
	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function WatchSomeoneElse()
	{
		PickSomePrey();
		if(InterestPawn != None)
			GotoStateSave('CheckOutPossiblePrey');
		else
			GotoStateSave('Thinking', 'TryAgain');
	}
	///////////////////////////////////////////////////////////////////////////////
	// Check for bumping the guy you want to mug
	///////////////////////////////////////////////////////////////////////////////
	event Bump( Actor Other )
	{
		local byte StateChange;

		// If you bump you're prey, just mug them now
		if(InterestPawn.Health > 0
			&& InterestPawn == Other)
		{
			PrepToMug();
			StateChange = 1;
		}

		if(StateChange != 1)
			DangerPawnBump(Other, StateChange);
	}
	///////////////////////////////////////////////////////////////////////////////
	// You've watched them enough for prelim stuff, decide to really stalk them or not
	///////////////////////////////////////////////////////////////////////////////
	function DecideToPickThem()
	{
		local bool bPicked;

		if(InterestPawn.bPlayer)
		{
			if(FRand() < PICK_PLAYER_FOR_MUGGING)
			{
				PickThem();
				bPicked=true;
			}
		}
		else
		{
			if(FRand() < PICK_NPC_FOR_MUGGING)
			{
				PickThem();
				bPicked=true;
			}
		}
		// We didn't pick them, look again
		if(!bPicked)
			WatchSomeoneElse();
	}

	///////////////////////////////////////////////////////////////////////////////
	// Start stalking this guy
	///////////////////////////////////////////////////////////////////////////////
	function PickThem()
	{
		statecount = 1 + Rand(STARE_COUNT);
		CurrentFloat = START_APPROACH_RADIUS;
		TimeWatchingPrey = 0.0;
//		GotoStateSave('WatchPrey');
		// Don't watch them at all, walk towards them and when you bump them
		// pick pocket them.
		SetEndGoal(InterestPawn, 1);
		SetNextState('CheckOutPossiblePrey');
		GotoStateSave('WalkToPrey');
	}
	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function BeginState()
	{
		PrintThisState();
		Focus = InterestPawn;
	}

Begin:
	Sleep(1.0);
KeepWatching:
	CheckPrey();
	// Stand around scope the area for someone to mug
	Sleep(PREY_CHECK_TIME);
	statecount--;
	if(statecount <= 0)
		DecideToPickThem();
	Goto('KeepWatching');
}
/*
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// We watch this person thinking about mugging them
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state WatchPrey extends CheckOutPossiblePrey
{
	///////////////////////////////////////////////////////////////////////////////
	// Make sure you have a line of sight to them
	///////////////////////////////////////////////////////////////////////////////
	function CheckPrey()
	{
		local float usedist;

		// If the interest dies, or becomes involved in a fight, lose interest in them
		if(PreyIsInvalid())
		{
			GotoStateSave('Thinking');
		}
		else
		{
			// Consider mugging them now
			if(TimeWatchingPrey > WATCH_PREY_MIN
				&& Frand() > (TimeWatchingPrey/WATCH_PREY_MAX))
				PrepToMug();
			// If you can't seem then, run closer
			else if(!FastTrace(InterestPawn.Location, MyPawn.Location))
			{
				SetEndGoal(InterestPawn, CurrentFloat);
				CurrentFloat=0.95*CurrentFloat;	// Bring the radius in some, so you can get
												// closer if this doesn't work
				if(CurrentFloat < DEFAULT_END_RADIUS)
					CurrentFloat = DEFAULT_END_RADIUS;
				SetNextState('WatchPrey');
				ScreamState=SCREAM_STATE_NONE; // clear scream state before we run
				GotoStateSave('RunToTarget');
			}
			// If they get too far away, walk or run to catch up to them
			else
			{
				// Check the distance to them
				usedist = VSize(InterestPawn.Location-MyPawn.Location);
				if(usedist > RUN_TO_PREY)
				{
					SetEndPoint(InterestPawn.Location, RUN_TO_PREY-MyPawn.CollisionRadius);
					SetNextState('WatchPrey');
					ScreamState=SCREAM_STATE_NONE; // clear scream state before we run
					GotoStateSave('RunToTarget');
				}
				else if(usedist > WALK_TO_PREY)
				{
					SetEndPoint(InterestPawn.Location, WALK_TO_PREY-MyPawn.CollisionRadius);
					SetNextState('WatchPrey');
					GotoStateSave('WalkToPrey');
				}
			}
		}
	}

Begin:
	Sleep(1.0);
KeepWatching:
	CheckPrey();
	// Stand around scope the area for someone to mug
	Sleep(PREY_CHECK_TIME);
	TimeWatchingPrey+=PREY_CHECK_TIME;
	Goto('KeepWatching');
}
*/

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// WalkToPrey
// Stalk the person you're interested in.
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state WalkToPrey extends WalkToTarget
{
	ignores RespondToTalker, PerformInterestAction, RatOutAttacker, CheckDeadBody, CheckDeadHead,
		WatchFunnyThing, CheckDesiredThing, CheckForIntruder, CanStartConversation, 
		FreeToSeekPlayer, CanBeMugged, ForceGetDown;
	///////////////////////////////////////////////////////////////////////////////
	// When you're bumped do the pick pocketing
	///////////////////////////////////////////////////////////////////////////////
	event Bump( Actor Other )
	{
		local byte StateChange;

		// If it's your alive prey, steal the money!
		if(InterestPawn.Health > 0
			&& InterestPawn == Other)
		{
			PrepToMug();
			StateChange = 1;
		}

		if(StateChange != 1)
			DangerPawnBump(Other, StateChange);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// DoMugging
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state DoMugging
{
	ignores DoWaitOnOtherGuy, SetupSideStep, SetupBackStep, SetupMoveForRunner, TryToSendAway,
		TryToGreetPasserby, DonateSetup, PerformInterestAction, CheckDesiredThing,
		StartConversation, RespondToQuestionNegatively, CheckDeadBody, CheckDeadHead, InterestIsAnnoyingUs, 
		GetHitByDeadThing, QPointSaysMoveUpInLine, SetToPickPocketPlayer, CanBeMugged,
		ForceGetDown, AllowOldState, PrepToMug;

	///////////////////////////////////////////////////////////////////////////////
	// Only hear our guy
	///////////////////////////////////////////////////////////////////////////////
	function GetReadyToReactToDanger(class<TimedMarker> dangerhere, 
									FPSPawn CreatorPawn, 
									Actor OriginActor,
									vector blipLoc,
									optional out byte StateChange)
	{
		if(CreatorPawn == Attacker
			|| CreatorPawn == InterestPawn)
			Super.GetReadyToReactToDanger(dangerhere, CreatorPawn, OriginActor, blipLoc, StateChange);
	}

	///////////////////////////////////////////////////////////////////////////////
	// See if we can see our guy
	///////////////////////////////////////////////////////////////////////////////
	function CheckObservePawnLooks(FPSPawn LookAtMe)
	{
		SetRotation(MyPawn.Rotation);

		if(LookAtMe == InterestPawn)
			ActOnPawnLooks(LookAtMe);
	}

	///////////////////////////////////////////////////////////////////////////////
	// Magically grant this guy cash
	///////////////////////////////////////////////////////////////////////////////
	function GrantThemCash(FPSPawn ThisPawn, int GiveAmount)
	{
		local Inventory thisinv;
		local P2PowerupInv pinv;
		local byte CreatedNow;

		thisinv = ThisPawn.CreateInventoryByClass(class'Inventory.MoneyInv', CreatedNow);

		pinv = P2PowerupInv(thisinv);

		// Add in what we gave them
		if(pinv != None)
		{
			pinv.AddAmount(GiveAmount);
			// set it as your item
			if(ThisPawn != None)
				ThisPawn.SelectedItem = pinv;
		}
	}

	///////////////////////////////////////////////////////////////////////////////
	// Go into their inv and grab their money out and give it to you
	///////////////////////////////////////////////////////////////////////////////
	function int TakeTheirMoney()
	{
		local int AmountTaken;
		local P2PowerupInv moreinv;

		moreinv = P2PowerupInv(InterestPawn.FindInventoryType(class'Inventory.MoneyInv'));

		if(moreinv != None)
		{
			AmountTaken = moreinv.Amount;
			// Take all they have
			moreinv.ReduceAmount(moreinv.Amount);
		}
		return AmountTaken;
	}

	///////////////////////////////////////////////////////////////////////////////
	// Check the interest once more to see if it's okay to try to mug them
	///////////////////////////////////////////////////////////////////////////////
	function FinalValidCheck()
	{
		local float usedist;
		local byte StateChange;

		usedist = VSize(InterestPawn.Location - MyPawn.Location);
		// If something is in the way, and you're already close enough, give up
		if(!FastTrace(MyPawn.Location, InterestPawn.Location))
		{
			if(usedist < (MUG_MIN_RADIUS + MyPawn.CollisionRadius))
			{
				GotoStateSave('Thinking');
				return;
			}
			// something's in the way, but we can get closer
			else
			{
				SetEndGoal(InterestPawn, 1);
				SetNextState('CheckOutPossiblePrey');
				GotoStateSave('WalkToPrey');
				return;
			}
		}
	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function SetupGrabMoney()
	{
		local MoneyPickup mp;
		local vector checkv;

		checkv = (MyPawn.Location + InterestPawn.Location)/2;
		foreach CollidingActors(class'MoneyPickup', mp, MUG_MAX_RADIUS, checkv)
		{
			if(mp != None)
				Focus = mp;
		}

		if(MoneyPickup(Focus) != None)
		{
			// Scan the area for the new money
			bDontSetFocus=true;
			SetEndGoal(Focus, TIGHT_END_RADIUS);
			SetNextState(GetStateName(), 'PickingUpMoney');
			ScreamState=SCREAM_STATE_NONE; // clear scream state before we run to the money
			GotoStateSave('RunToTarget');
		}
	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function BeginState()
	{
		PrintThisState();
		//if(InterestPawn != None)
		//	SetAttacker(InterestPawn);
		MyPawn.StopAcc();
		Focus = InterestPawn;
	}
	function EndState()
	{
		InterestPawn = None;
		bImportantDialog=false;
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// DoNonPlayerMugging, mug AI characters
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state DoNonPlayerMugging extends DoMugging
{
Begin:
	FinalValidCheck();
	AWBystanderController(InterestPawn.Controller).SetupGettingPickPocketed(MyPawn);
	bImportantDialog=true;
	// Pat guy down
	MyPawn.PlayHelloGesture(2.0);
	// talk while doing it
	Sleep(0.5);
	PrintDialogue("Sorry about bumping you.");
	/////////////////SayTime = Say(MyPawn.myDialog.lApologizePickPocket, true);
	SayTime = Say(MyPawn.myDialog.lApologize, true);
	Sleep(0.5);
	// Give NPC money
	GrantThemCash(InterestPawn, Rand(CASH_GIVE_RAND) + CASH_GIVE_BASE);
	// Take money from NPC's inventory
	GrantThemCash(MyPawn, TakeTheirMoney());
	Sleep(0.5);

RunningAway:
	Sleep(0.1);
	if(!PickRandomDest())
		Goto('RunningAway');	// Didn't find a valid point, try again
	SetNextState('Thinking');
	ScreamState=SCREAM_STATE_NONE; // clear scream state before we run
	GotoStateSave('RunToTarget');
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// DoPlayerMugging
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state DoPlayerMugging extends DoMugging
{
	///////////////////////////////////////////////////////////////////////////////
	// Take dude money
	///////////////////////////////////////////////////////////////////////////////
	function TakeDudeMoney()
	{
		local int AmountTaken;
		local P2PowerupInv moreinv;

		moreinv = P2PowerupInv(InterestPawn.FindInventoryType(class'Inventory.MoneyInv'));

		// Check to make sure he still has money
		if(moreinv == None
			|| moreinv.Amount == 0)
		{
			// Get mad becuase he doesn't have any money to take!
			GotoState('DoPlayerMugging', 'NoDudeMoney');
		}
		else	// Take his money
			GrantThemCash(MyPawn, TakeTheirMoney());
	}

	///////////////////////////////////////////////////////////////////////////////
	// Check the interest once more to see if it's okay to try to mug them
	///////////////////////////////////////////////////////////////////////////////
	function FinalValidCheck()
	{
		// If the player is engaged with a cashier, don't mess with him
		if(InterestPawn != None
			&& P2Player(InterestPawn.Controller).CanBeMugged(MyPawn))
			Super.FinalValidCheck();
		else
			GotoStateSave('Thinking');
	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function SwitchPlayerToMoney()
	{
		P2Player(InterestPawn.Controller).SwitchToThisPowerup(class'MoneyInv'.default.InventoryGroup,
														class'MoneyInv'.default.GroupOffset);
	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	function EndState()
	{
		// Unhook us from the player if we exited early for something
		// like fighting
		if(Attacker != None)
		{
			if(InterestPawn != None
				&& P2Player(InterestPawn.Controller) != None)
				P2Player(InterestPawn.Controller).UnhookPlayerGetMugged();
		}
		Super.EndState();
	}

Begin:
	FinalValidCheck();
	P2Player(InterestPawn.Controller).SetupGettingMugged(MyPawn);
	bImportantDialog=true;
	// Pat guy down
	MyPawn.PlayHelloGesture(2.0);
	// talk while doing it
	Sleep(0.5);
	SwitchPlayerToMoney();
	PrintDialogue("Sorry about bumping you.");
	/////////////////SayTime = Say(MyPawn.myDialog.lApologizePickPocket, true);

	SayTime = Say(MyPawn.myDialog.lApologize, true);
	Sleep(1.0);
	// Take money from dude's inventory
	TakeDudeMoney();
	Sleep(0.5);

RunningAway:
	Sleep(0.1);
	if(!PickRandomDest())
		Goto('RunningAway');	// Didn't find a valid point, try again
	// Make dude ready to realize what happened
	AWPlayer(InterestPawn.Controller).FinishPickPocket();

	SetNextState('Thinking');
	ScreamState=SCREAM_STATE_NONE; // clear scream state before we run
	GotoStateSave('RunToTarget');

NoDudeMoney:
	// Mad becuase he dropped the money or something at the last second
	SayTime = Say(MyPawn.myDialog.lwhatthe, true);
	Sleep(SayTime);
	GotoStateSave('Thinking');
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// ShootAtAttacker
// Mugger is special.. if he gets to the point of calling CheckToMoveAround,
// then he's completed a full attack sequence, therefore--he's ready to cowardly
// run away! Lower PainThreshold to make him run away early after getting hurt.
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state ShootAtAttacker
{
	///////////////////////////////////////////////////////////////////////////
	// Don't do anything strategic here.. just run away
	///////////////////////////////////////////////////////////////////////////
	function CheckToMoveAround()
	{
		if(PickRandomDest())
		{
			SetNextState('Thinking');
			ScreamState=SCREAM_STATE_NONE; // clear scream state before we run
			GotoStateSave('RunToTarget');
		}
	}
	///////////////////////////////////////////////////////////////////////////////
	// Don't do anything strategic here.. just run away
	///////////////////////////////////////////////////////////////////////////////
	function PerformStrategicMoves(optional bool bForce, optional bool bForceBackUp, optional out byte StateChange)
	{
		if(PickRandomDest())
		{
			SetNextState('Thinking');
			ScreamState=SCREAM_STATE_NONE; // clear scream state before we run
			GotoStateSave('RunToTarget');
		}
	}
}

defaultproperties
{
     SwitchWeaponFreq=0.000000
}
