///////////////////////////////////////////////////////////////////////////////
// AWZombie
// Copyright 2004 Running With Scissors, Inc.  All Rights Reserved.
//
// Base class for all zombie characters in AW.
//
///////////////////////////////////////////////////////////////////////////////
class AW7Zombie extends AWZombie
	notplaceable;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
var(Footsteps) Sound FootStepSounds[5];
var(Footsteps) bool bDoFootsteps;

var(Character) MeshAnimation AW7Anim;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
var float StuckTime;				// How long the player has been 'stuck'. Determined below. If they are not
									// moving in Z for too long (MAX_STUCK_TIME),
									// then warp them to the nearest good pathnode.
var float StuckCheckRadius;			// Size around which you check for pathnodes to warp to
var PathNode LastUnstuckPoint;		// Save the last point we warped to, after being unstuck. Don't use it
									// the very next time, to keep us from infinite unsticking--still stuck
									// scenarios.
var bool bPlayingFootstepSound;	// Set to true when you play a step, false after timer is
								// called again
								
var bool bAltZombieAnims;			// Set to true if we use the alternate ED zombie anims.

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
const MAX_STUCK_TIME				=	2.0;
const STUCK_RADIUS					=	2048;

const RUN_FOOTSTEP_TIME		=	0.35;
const WALK_FOOTSTEP_TIME	=	0.60;

const WALK_VOL				= 0.07;
//=	0.07;
const RUN_VOL				= 0.2;
//=	0.2;

const FOOTSTEP_RADIUS			=	100;
const FOOTSTEP_RADIUS_MP		=	200;
const FOOTSTEP_RADIUS_LOCAL_MP	=	20;

// ED BLOOD CODE
///////////////////////////////////////////////////////////////////////////////
//	Tries to make a small blood splat on the ground based on hit velocity
///////////////////////////////////////////////////////////////////////////////
function DripBloodOnGround(vector Momentum)
{
	local Rotator NewRot;
	local Actor HitActor;
	local vector checkpoint, HitNormal, HitLocation;

	//log("before mom "$Momentum);
	Momentum.x*=FRand();
	Momentum.y*=FRand();
	Momentum = Normal(Momentum);
	Momentum.z=BLOOD_DRIP_GRAVITY;
	//log("after mom "$Momentum);
	checkpoint = Location + DRIP_FLOOR_Z_CHECK*Momentum;
	HitActor = Trace(HitLocation, HitNormal, checkpoint, Location, true);
	if ( HitActor != None
		&& HitActor.bWorldGeometry 
		&& HitNormal.z > 0.9)
	{
		NewRot = Rotator(-HitNormal);
		NewRot.Roll=(65536*FRand());
		spawn(class'EDBloodDripSplat',self,,HitLocation,NewRot);
	}
}

///////////////////////////////////////////////////////////////////////////////
//	A blood splash here
///////////////////////////////////////////////////////////////////////////////
function BloodHit(vector BloodHitLocation, vector Momentum)
{
	local vector BloodOffset, dir, HitLocation, HitNormal, checkpoint;
	local float tempf;
	local EDBloodImpactMachineGun bloodhitvar;
	local Rotator NewRot;
	local Actor HitActor;
	//, Mo;
	//local class<Effects> DesiredEffect;
//		class<P2Damage>(damageType).static.GetPawnDamageEffect(HitLocation, Damage, Momentum, self, (Level.bDropDetail || !Level.bHighDetailMode));
//		DesiredEffect = DamageType.static.GetPawnDamageEffect(HitLocation, Damage, Momentum, self, (Level.bDropDetail || !Level.bHighDetailMode));

//		if ( DesiredEffect != None )
//		{

	// Find direction to center
	dir = BloodHitLocation - Location;
	dir = Normal(BloodHitLocation - Location);
	// push it away from the his center
	BloodOffset = 0.2 * CollisionRadius * dir;
	// pull it up some from the bottom and pull it down from the top
	BloodOffset.Z = BloodOffset.Z * 0.75;

	//Mo = Momentum;
	//if ( Mo.Z > 0 )
		//Mo.Z *= 0.5;

//			spawn(DesiredEffect,self,,BloodHitLocation + BloodOffset);//, rotator(Mo));
//		}
	////////////////
	// Blood that squirts in the air
	bloodhitvar = spawn(class'EDBloodImpactMachineGun',self,,BloodHitLocation+BloodOffset);
	// This makes the blood shoot off at cool angles

	////////////////
	// Blood that shoots onto the wall 
	// Check to see if you're close enough to the wall, to squirt blood on it.
	// Do this by coming out of the actor where we hit and continue along the path
	// that goes from the original hit point, toward the player. (So look 
	// behind the player)
	checkpoint = BloodHitLocation + DIST_TO_WALL_FOR_BLOODSPLAT*Normal(Momentum);
	//log("momentum "$Momentum);
	HitActor = Trace(HitLocation, HitNormal, checkpoint, BloodHitLocation, true);

	if ( HitActor != None
		&& HitActor.bStatic ) 
//	if(LevelInfo(HitActor) != None
//		|| TerrainInfo(HitActor) != None
//		|| StaticMeshActor(HitActor) != None
//		|| Brush(HitActor) != None)
	{
		NewRot = Rotator(-HitNormal);
		/*
		tempf = atan(Momentum.x/Momentum.y);
			log("tempf "$tempf);
		NewRot.Roll = 65536*(atan(Momentum.y/Momentum.x)/(2*Pi));
		if(Momentum.x < 0)
			NewRot.Roll -= 32768;

  maybe something with the wall normal
		NewRot.Roll = NewRot.Roll & 65535;
		spawn(class'BloodSplatOblique',self,,HitLocation,NewRot);
		*/
		NewRot.Roll=(65536*FRand());
		spawn(class'EDBloodMachineGunSplat',self,,HitLocation,NewRot);
	}

	////////////////
	// Drips of blood on the ground around you (smaller)
	if(FRand() <= 0.7)
	{
		DripBloodOnGround(Momentum);
	}
}
// END ED BLOOD CODE

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function PostBeginPlay()
{
	Super.PostBeginPlay();

	StuckCheckRadius = STUCK_RADIUS;
}

///////////////////////////////////////////////////////////////////////////////
// Link this pawn to the anims it needs
///////////////////////////////////////////////////////////////////////////////
simulated function LinkAnims()
{
	// Also put in AW7 anims.
	LinkSkelAnim(AW7Anim);

	// Also put in more AW single player anims.
	LinkSkelAnim(AW_SPMeshAnim);

	// Links to the special specified mesh for SP games.
	LinkSkelAnim(GetDefaultAnim(SkeletalMesh(Mesh)));

	// Always link to the core anims, too, because some characters use a mixture
	// of their own anims plus some core anims.  Linking to core anims twice,
	// which can happen if default anims happen to match core anims, is safe.
	LinkSkelAnim(CoreMeshAnim);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function SetupAnims()
{
	LinkAnims();

	if(AnimGroupUsed != -1)
		AnimGroupUsed = Rand(10);	// Pick which walks and other random anims to pick.
	else
		AnimGroupUsed=0;	// Pick the basic one
		
	if (FRand() < 0.5)
		bAltZombieAnims = True;

	if(bFloating)
		MovementAnims[0]	= 'z_bob';
	else if(AnimGroupUsed == 0)
		MovementAnims[0]	= 'z_walk1';
	else if(AnimGroupUsed == 1)
		MovementAnims[0]	= 'z_walk2';
	else if(AnimGroupUsed == 2)
		MovementAnims[0]	= 'z_walk3';
	else if(AnimGroupUsed == 3)
		MovementAnims[0]	= 'z_walk4';
	else if(AnimGroupUsed == 4)
		MovementAnims[0]	= 'z_walk5';
	else if(AnimGroupUsed == 5)
		MovementAnims[0]	= 'z_walk6';
	else if(AnimGroupUsed == 6)
		MovementAnims[0]	= 'z_walk7';
	else if(AnimGroupUsed == 7)
		MovementAnims[0]	= 'z_walk8';
	else if(AnimGroupUsed == 8)
		MovementAnims[0]	= 'z_walk9';
	else if(AnimGroupUsed == 9)
		MovementAnims[0]	= 'z_walk10';

	TurnLeftAnim		= MovementAnims[0];
	TurnRightAnim		= MovementAnims[0];
	MovementAnims[0]	= MovementAnims[0];
	MovementAnims[1]	= MovementAnims[0];
	MovementAnims[2]	= MovementAnims[0];
	MovementAnims[3]	= MovementAnims[0];


	// After all the setup, check to see if we're too fat to run much
	if (bIsFat)
	{
		// He's fat, so drop his fitness
		if(default.Fitness != Fitness)
			Fitness=0.5*Fitness;
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function SetAnimWalking()
{
	if(bFloating)
		MovementAnims[0]	= 'z_bob';
	else if(AnimGroupUsed == 0)
		MovementAnims[0]	= 'z_walk1';
	else if(AnimGroupUsed == 1)
		MovementAnims[0]	= 'z_walk2';
	else if(AnimGroupUsed == 2)
		MovementAnims[0]	= 'z_walk3';
	else if(AnimGroupUsed == 3)
		MovementAnims[0]	= 'z_walk4';
	else if(AnimGroupUsed == 4)
		MovementAnims[0]	= 'z_walk5';
	else if(AnimGroupUsed == 5)
		MovementAnims[0]	= 'z_walk6';
	else if(AnimGroupUsed == 6)
		MovementAnims[0]	= 'z_walk7';
	else if(AnimGroupUsed == 7)
		MovementAnims[0]	= 'z_walk8';
	else if(AnimGroupUsed == 8)
		MovementAnims[0]	= 'z_walk9';
	else if(AnimGroupUsed == 9)
		MovementAnims[0]	= 'z_walk10';

	TurnLeftAnim		= MovementAnims[0];
	TurnRightAnim		= MovementAnims[0];
	MovementAnims[0]	= MovementAnims[0];
	MovementAnims[1]	= MovementAnims[0];
	MovementAnims[2]	= MovementAnims[0];
	MovementAnims[3]	= MovementAnims[0];
}

simulated function SetAnimRunning()
{
	if(bFloating)
		MovementAnims[0]	= 'c_walk';
	else if (bAltZombieAnims)
		MovementAnims[0]	= 'z_charge_alt';
	else
		MovementAnims[0]	= 'z_charge';
	TurnLeftAnim		= MovementAnims[0];
	TurnRightAnim		= MovementAnims[0];
	MovementAnims[0]	= MovementAnims[0];
	MovementAnims[1]	= MovementAnims[0];
	MovementAnims[2]	= MovementAnims[0];
	MovementAnims[3]	= MovementAnims[0];
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function name GetAnimStand()
{
	if(bFloating)
		return 'z_bob';
	else if (bAltZombieAnims)
		return 'z_idle_stand_alt';
	else
		return 'z_idle_stand';
}

simulated function name GetAnimSwipeLeft()
{
	if(!bFloating
		&& bMissingLegParts)
	{
		if (bAltZombieAnims)
			return 'z_crawlswipeleftalt';
		else
			return 'z_crawlswipeleft';
	}
	else if (bAltZombieAnims)
		return 'z_swipeleft_alt';
	else
		return 'z_swipeleft';
}

simulated function name GetAnimSwipeRight()
{
	if(!bFloating
		&& bMissingLegParts)
	{
		if (bAltZombieAnims)
			return 'z_crawlswiperightalt';
		else
			return 'z_crawlswiperight';
	}
	else if (bAltZombieAnims)
		return 'z_swiperight_alt';
	else
		return 'z_swiperight';
}

simulated function name GetAnimSmash()
{
	if (bAltZombieAnims)
		return 'z_smash_alt';
	else
		return 'z_smash';
}

simulated function name GetAnimVomitAttack()
{
	if(!bFloating
		&& bMissingLegParts)
	{
		if (bAltZombieAnims)
			return 'z_crawlspit_alt';
		else
			return 'z_crawlspit';
	}
	else if (bAltZombieAnims)
		return 'z_spit_alt';
	else
		return 'z_spit';
}

simulated function name GetAnimDeathCrawl()
{
	if (bAltZombieAnims)
		return 'z_crawl_alt';
	else
		return 'z_crawl';
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function SetupBoltons()
{
	local int i;

	Super.SetupBoltons();

	for (i=0; i<ArrayCount(Boltons); i++)
	{
		if (Boltons[i].Part != None && Boltons[i].bAttachToHead)
			Boltons[i].Part.SetDrawScale3d(MyHead.DrawScale3D);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Decide on a prize for a quick kill
///////////////////////////////////////////////////////////////////////////////
function PickQuickKillPrize(int KillIndex)
{
	local class<Inventory> invclass;
	local Inventory newone;

	switch(KillIndex)
	{
		case 0:
			invclass = class'DonutInv';
		break;
		case 1:
			invclass = class'MoneyInv';
		break;
		case 2:
			invclass = class'FastFoodInv';
		break;
		case 3:
			invclass = class'MaDBeerInv';
		break;
		case 4:
			invclass = class'CrackInv';
		break;
		case 5:
			invclass = class'MaDWeedInv';
		break;
		case 6:
			invclass = class'CatnipInv';
		break;
	}
	newone = spawn(invclass, self);
	TossThisInventory(vector(Rotation), newone);
}

///////////////////////////////////////////////////////////////////////////////
// toss out the inventory passed in
///////////////////////////////////////////////////////////////////////////////
function bool TossThisInventory(vector TossVel, Inventory ThisInv)
{
	local vector X,Y,Z;
	local Rotator userot;
	local P2Weapon ThisWeap;

	if(ThisInv == None)
		return false;

	// Really make it stop doing it's stuff
	// And don't drop things that can't be dropped
	ThisWeap = P2Weapon(ThisInv);
	if(ThisWeap != None)
	{
		if(!ThisWeap.bCanThrow
			|| (!ThisWeap.bCanThrowMP
				&& !Level.Game.bIsSinglePlayer))
			return false;

		ThisWeap.ForceEndFire();
	}

	// Comment out the 1409 fix
/*
	if(P2PowerupInv(ThisInv) != None)
	{
		if(!P2PowerupInv(ThisInv).bCanThrow
			// 1409 fix, make sure you have stuff to throw!
			// Before this, you could use a crack pipe, then throw it quickly as the
			// smoke was coming out, and you'd still get the health and the crack pipe back--fixed!
			|| P2PowerupInv(ThisInv).Amount <= 0)
			return false;
	}
*/

	ThisInv.velocity = TossVel;
	userot = Rotation;
	userot.Yaw = FRAnd()*65535;
	GetAxes(userot,X,Y,Z);
	// And make the ThisWeap align with the direction of the throw
	ThisInv.SetRotation(userot);
	ThisInv.DropFrom(Location + 0.3 * CollisionRadius * X + CollisionRadius * Z);
	return true;
}

///////////////////////////////////////////////////////////////////////////////
// Decide if he's stuck or not
///////////////////////////////////////////////////////////////////////////////
function bool DetectStuckPlayer(float DeltaTime)
{
	// If you're falling and not moving for too long, you're stuck
	if(Physics == PHYS_Falling
		&& !bIsCrouched)
	{
		// If you're not moving in z for too long
		if(Velocity.z == 0)
		{
			StuckTime += DeltaTime;

			// Now that he's really stuck, move him
			if(StuckTime > MAX_STUCK_TIME)
			{
				StuckTime = 0;
				return true;
			}
		}
		else
			StuckTime = 0;
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////
// Find the nearest free pathnode to the stuck player, and put him there
// Warps him to each possible spot to make sure it will work. But it doesn't
// warp him back, it just uses the oldloc to test from. The last one you warped
// him to will have to be the closest to oldloc.
///////////////////////////////////////////////////////////////////////////////
function HandleStuckPlayer()
{
	local PathNode pn, savepn;
	local float closedist, usedist;
	local vector useloc, oldloc;

	Warn(self$" ERROR! NPC WAS STUCK here: "$Location);

	oldloc = Location;

	closedist = StuckCheckRadius;
	// Check pathnodes in a given radius
	foreach RadiusActors(class'Pathnode', pn, StuckCheckRadius, oldloc)
	{
		usedist = VSize(pn.Location - oldloc);
		if(LastUnstuckPoint	!= pn		// Not where we last unstuck ourself from
			&& usedist < closedist		// closest one
			&& !pn.bBlocked)			// not blocked
		{
			// Warp the player there now, to make sure the spot was
			// okay for him to be there. If this happens, it will only
			// work with the closest node, so this will also be the final move
			useloc = pn.Location;
			// Add just enough buffer to pathnode, to make sure he somehow doesn't
			// get warped to a point below the floor, and then fall through the floor.
			useloc.z += (CollisionHeight/2);
			if(SetLocation(useloc))
			{
				closedist = usedist;
				savepn = pn;
			}
		}
	}

	if(savepn != None)
	{
		//mypawnfix
		StuckCheckRadius = STUCK_RADIUS;
		LastUnstuckPoint = savepn;	// Save where we last unstuck from so as to not use it again, next time
		Warn(self$" UNsticking NPC--warping him to "$savepn);
	}
	else
	{
		Warn(self$" UNsticking warp failed");
		// You've failed, then increase you're check area
		StuckCheckRadius += STUCK_RADIUS;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Take damage (pawn call) but also save who attacked me
// Take limbs off in AW.
///////////////////////////////////////////////////////////////////////////////
function TakeDamage( int Damage, Pawn instigatedBy, Vector hitlocation,
						Vector momentum, class<DamageType> damageType)
{
	local bool bUseSuper;
	local int actualDamage, StartDamage;
	local Controller Killer;
	local byte StateChange;
	local LambController lambc;

	lambc = LambController(Controller);

	//log(self$" take damage "$damagetype$" takes "$TakesZombieSmashDamage);
	// If we don't have a controller then we're either the player in a movie, or
	// we're an NPC starting out in a pain volume--either way, we don't
	// want to take damage in this state, without a controller.
	// For the player, this gives him god mode, while a movie is playing.
	if(Controller == None)
		return;

	if(!AcceptHit(hitlocation, momentum))
		return;

	StartDamage = Damage;
	DamageInstigator = instigatedBy;

	// Wake them from stasis now that we've been hit
	if(Controller.bStasis)
		lambc.ComeOutOfStasis(false);

	if(ClassIsChildOf(damageType, class'SuperShotgunDamage'))
		Damage = TakesScytheDamage*Damage;

	if(// Can't cut off player limbs
		P2Player(Controller) == None)
	{
		// If it's a damage type and hasn't been lowered, allow limb severance
		// but if TakesMacheteDamage (for instance) is less than 1.0, then don't let limbs be cut
		if(ClassIsChildOf(damageType, class'SuperShotgunDamage'))
		{
			if(Damage >= StartDamage)
				bUseSuper = !(HandleScythe(InstigatedBy, momentum, damagetype, Damage, HitLocation));
			else // convert to lower damage type
				damageType = class'ShotgunDamage';
		}
		else
			bUseSuper=true;

	}
	else // Make sure the player picks the super take damage
		bUseSuper=true;

	if(!bUseSuper)
	{
		// Multiply damage as necessary per pawn
		//Damage = FPSPawn(InstigatedBy).DamageMult*Damage;

		// Modify as necessary per game
		actualDamage = Level.Game.ReduceDamage(Damage, self, instigatedBy, HitLocation, Momentum, DamageType);

		// Save the type that just hurt us
		LastDamageType = class<P2Damage>(DamageType);

		// Armor check.
		// Intercept damage, and if you have armor on, and it's a certain type of damage, 
		// modify the damage amount, based on the what hurt you.
		// Armor doesn't do anything for head shots
		if(Armor > 0
			&& bHasHead
			&& (Controller == None
				|| !Controller.bGodMode))
			ArmorAbsorbDamage(instigatedby, actualDamage, DamageType, HitLocation);

		// Don't call at all if you didn't get hurt
		// The above function does a better check on the head, so you may have shot too high or
		// something, at which point, it'll set the damage to 0, causing this to exit early
		if(Damage <= 0)
		{
			// Tell the character about the non-damage. Most of them will ignore this damage
			// but some people (like Krotchy) will use this to do things
			// Report the original damage asked to be delivered as a negative, so it's not
			// used as actual damage, but it's used to know how bad the damage would have been.
			if ( Controller != None )
				Controller.NotifyTakeHit(instigatedBy, HitLocation, -StartDamage, DamageType, Momentum);
			return;
		}

		// Send the real momentum to this function, please
		PlayHit(actualDamage, hitLocation, damageType, Momentum);

		// Take off health from damage
		Health = Health - actualDamage;

		// Check if he's dead
		if ( Health <= 0 )
		{
			// pawn died
			if ( instigatedBy != None )
				Killer = InstigatedBy.GetKillerController();
			if ( bPhysicsAnimUpdate )
				TearOffMomentum = Momentum / Mass;
//			if (DamageType.Default.GibModifier >= 100)
//				BlowOffHeadAndLimbs(InstigatedBy, momentum, Damage);

			Died(Killer, damageType, HitLocation);
		}
		else
		{
			// Don't make things shoot you up into the air unless it's specific damage types
			if(class<P2Damage>(damageType) == None
				|| !class<P2Damage>(damageType).default.bAllowZThrow)
			{
				if(Physics == PHYS_Walking)
					momentum.z=0;
			}
			AddVelocity( momentum );

			// Tell the character about the damage
			if ( Controller != None )
				Controller.NotifyTakeHit(instigatedBy, HitLocation, Damage, DamageType, Momentum);
		}
	}
	else
	{
//		if (Damage >= Health && DamageType.Default.GibModifier >= 100)
//			BlowOffHeadAndLimbs2(InstigatedBy, BlowOffMom, Damage);

		Super.TakeDamage(Damage, InstigatedBy, HitLocation, momentum, damageType);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Chop person in half
//
// Doesn't check if they still have a whole body to slice in half--just
// does it anyway, so be careful! Check bMissingTop/BottomHalf first.
///////////////////////////////////////////////////////////////////////////////
function ChopInHalf(Pawn InstigatedBy, out vector momentum, out int Damage,
					out vector TrueHitLoc)
{
	local AWPerson bottomh;
	local coords usecoords;
	local rotator userot;
	local float Animf, AnimR;
	local name CurrAnim;
	local Actor HitActor;
	local vector HitLocation, HitNormal, checkpoint;
	local PawnExplosion exp;
	local bool bTopHalfStartedAlive;

	// Mark the bottom as missing becuase we're turning this bone into
	// the top half, and your missing your legs
	bMissingBottomHalf=true;
	bMissingLegParts=true;
	bMissingLimbs=true;

	// Save if we started alive or not for the bottom half to know how to animate if no ragdoll is around
	if(Health > 0)
		bTopHalfStartedAlive=true;

	// Make sure it kills them
	Damage = Health;

	// Do effects
	PlaySound(CutInHalfSound,,,,,GetRandPitch());
	spawn(class'LimbExplode',self,,Location);

	// Shrink bottom half
	// Shrink everything down all over, to very small. This number
	// must be greater than 0, or the matrices for the bones will be lots
	SetBoneScale(TORSO_INDEX, 1/SHRINK_PELVIS, BONE_PELVIS);
	// Grow it back up, but only for the top of the body
	SetBoneScale(TORSO_INDEX+1, SHRINK_PELVIS, TOP_TORSO);
	// Pop up the top half some
	momentum = VRand();
	momentum.z=0;
	momentum = (TopMomMag)*Normal(momentum);
	momentum.z += TopMomMag;
	TrueHitLoc.z+=(CollisionHeight/2); // make it hit above the center some for some spin
	// Turn off your collision
	SetCollision(false, false, false);
	// Make the bottom half, starting with the proper orientation
	usecoords = GetBoneCoords(BONE_PELVIS);
	userot = Rotator(usecoords.YAxis);
	userot.Yaw-=32768;
	// Check if they are too close to the ground to spawn
	checkpoint = Location;
	checkpoint.z-=CollisionHeight;
	HitActor = Trace(HitLocation, HitNormal, checkpoint, Location, true);
	if(HitActor != None)
	{
		// Move it up, off the ground
		HitLocation.z+=(1.5*CollisionHeight);
	}
	else // it's find where it is
		HitLocation = Location;

	// Add stumps for top half
	HalfTorsoStumps();

	// Spawn the bottom half
	bottomh = spawn(class,Owner,,HitLocation);
	if(bottomh != None)
	{
		bottomh.bBottomHalfNewSpawn=true;
		// Remove head
		bottomh.DestroyHead();
		// Move bolt-ons from the bottom
		bottomh.DestroyBoltons();
		// Set mesh as the same
		bottomh.SetMyMesh(Mesh);
		// Make sure the bottom looks the same as the top
		bottomh.Skins[0] = Skins[0];
		// Set our clothing now that our skin is set, reset them first, then recheck them in the functions
		bottomh.bSkirt=false;
		bottomh.bPants=false;
		bottomh.DetermineClothing(GetItemName(String(Skins[0])));
		bottomh.DetermineClothing(GetItemName(String(Mesh)));
		// Get current anim
		GetAnimParams(0, CurrAnim, AnimF, AnimR);
		bottomh.ShouldCower(bIsCowering);
		bottomh.ShouldCrouch(bIsCrouched);
		bottomh.ShouldDeathCrawl(bIsDeathCrawling);
		// Play anim same as current one, but make it start at the end
		bottomh.PlayAnimAt(CurrAnim, 1.0, 0.0, 0, 1.0);
		if(!bTopHalfStartedAlive)
			bottomh.bBottomWarpDeathAnim=true;
		// Shrink top half
		bottomh.SetBoneScale(TORSO_INDEX, 0.0, TOP_TORSO);
		// Kill it to turn it into karma/animation
		bottomh.Health=0;
		bottomh.OldTopHalfRotationKarma=userot;
		bottomh.OldTopHalfRotationAnim=Rotation;
		bottomh.PlayDying(class'ScytheDamage', Location);
		// And you're the bottom, that's missing it's top, including your arms
		bottomh.bMissingTopHalf=true;
		bottomh.bMissingLimbs=true;
		bottomh.BoneArr[LEFT_ARM]=0;
		bottomh.BoneArr[LEFT_ARM+1]=0;
		bottomh.BoneArr[RIGHT_ARM]=0;
		bottomh.BoneArr[RIGHT_ARM+1]=0;

		// Add stumps for bottom half
		HalfPelvisStumps(bottomh);

		// set the ragdoll in motion
		bottomh.TakeDamage(Damage, InstigatedBy, TrueHitLoc, Momentum, class'P2Damage');
	}
	else // couldn't make the bottom, so make some explosion effects to show something happened
	{
		exp = spawn(class'PawnExplosion',,,usecoords.origin);
		exp.FitToNormal(usecoords.YAxis);
		// Check if the body had any missing legs
		// We didn't make the bottom half, but we still want to remove the stumps,
		// sending in None for our bottom half will do that for us.
		TransferStump(LEFT_LEG, class'Stump'.default.LeftLegI, None);
		TransferStump(RIGHT_LEG, class'Stump'.default.RightLegI, None);
	}

	// Make sure your legs are really gone too
	// Do this later, because the bottom half needs to know about missing limbs
	BoneArr[LEFT_LEG]=0;
	BoneArr[LEFT_LEG+1]=0;
	BoneArr[RIGHT_LEG]=0;
	BoneArr[RIGHT_LEG+1]=0;
	// Turn collision back on
	SetCollision(true, false, false);
	// Tell the dude if he did it
	if(AWDude(InstigatedBy) != None)
		AWDude(InstigatedBy).CutHalf(self);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
auto simulated state Living
{
	event Tick(float DeltaTime)
	{
		Super.Tick(DeltaTime);

		// Unsticking logic for NPCs.
        if (!Controller.bIsPlayer && DetectStuckPlayer(DeltaTime))
			HandleStuckPlayer();
	}

	///////////////////////////////////////////////////////////////////////////////
	// Check to change footstep speed
	///////////////////////////////////////////////////////////////////////////////
	simulated function SetWalking(bool bNewIsWalking)
	{
		local bool OldWalking;

		OldWalking = bIsWalking;

		Super.SetWalking(bNewIsWalking);

		if ( bNewIsWalking != OldWalking )
		{
			Timer();
		}
	}

	///////////////////////////////////////////////////////////////////////////////
	//  Play sound
	///////////////////////////////////////////////////////////////////////////////
	simulated function Timer()
	{
		local float vol, usepitch, userad;
		local vector usevel;
		local bool bOtherMp; // someone other than you

//		log(self$" timer, phys "$Physics$" walking "$bIsWalking$" velocity "$Velocity$" role "$Role$" viewport "$ViewPort(PlayerController(Controller).Player));

		// only on client or stand alone
		// don't play footsteps during cinema
		if(bDoFootsteps && !P2GameInfo(Level.Game).IsCinematic())
		{
			bPlayingFootstepSound=false;

			if ( Physics == PHYS_Walking )
			{
				usepitch=1.0;
				if(bIsWalking
					|| bIsCrouched)
				{
					vol = WALK_VOL;
					SetTimer(WALK_FOOTSTEP_TIME, false);
				}
				else
				{
					vol = RUN_VOL;
					SetTimer(RUN_FOOTSTEP_TIME, false);
				}

				usepitch=1.0;
				userad = FOOTSTEP_RADIUS;

				if(VSize(Velocity) > 0)
				{
					PlaySound(FootStepSounds[Rand(ArrayCount(FootStepSounds))],SLOT_Interact,vol,,userad,usepitch);
					bPlayingFootstepSound=true;
				}
			}
			else
				SetTimer(WALK_FOOTSTEP_TIME, false);
		}
	}
	simulated function BeginState()
	{
//		log(self$" beginstate Living, role "$Role$" remote "$RemoteRole);
		SetTimer(RUN_FOOTSTEP_TIME, false);
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Dying
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Dying
{
	///////////////////////////////////////////////////////////////////////////////
	// Get hurt, plus some ragdoll
	//
	// Only let you hurt the dead guys in single player, because in MP, the characters
	// won't match up with other people's computers and if you set someone on fire
	// they will be on fire in two different incorrect places (blood would shoot
	// out of weird places too.. ) It just makes a big mess.
	///////////////////////////////////////////////////////////////////////////////
	function TakeDamage( int Damage, Pawn InstigatedBy, Vector Hitlocation,
							Vector Momentum, class<DamageType> DamageType)
	{
		local bool bUseSuper;

		if (MyBodyFire == None)
			MyBodyFire = None;

		if(Level.Game != None
			&& Level.Game.bIsSinglePlayer)
		{
			if(class'P2Player'.static.BloodMode()
				// can't cut off player limbs
				&& P2Player(Controller) == None)
			{
				if(ClassIsChildOf(damageType, class'SuperShotgunDamage'))
					bUseSuper = !(HandleScythe(InstigatedBy, momentum, damagetype, Damage, HitLocation));
				else
					bUseSuper=true;
			}

			if(bUseSuper)
			{
				// Check to cut off limbs, and pass it along to original TakeDamage
				// for momentum transfer
				if(class'P2Player'.static.BloodMode()
					&& ClassIsChildOf(damageType, class'ExplodedDamage'))
					HandleExplosionDead(Damage, InstigatedBy, HitLocation, momentum, damageType);

				Super.TakeDamage(Damage, InstigatedBy, HitLocation, Momentum, DamageType);
			}
		}
	}
}

defaultproperties
{
	ControllerClass=Class'AW7ZombieController'
	bDoFootsteps=True
	FootStepSounds(0)=Sound'MoreMiscSounds.loudfootsteps.footstep1h'
	FootStepSounds(1)=Sound'MoreMiscSounds.loudfootsteps.footstep2h'
	FootStepSounds(2)=Sound'MoreMiscSounds.loudfootsteps.footstep4h'
	FootStepSounds(3)=Sound'MoreMiscSounds.loudfootsteps.footstep5h'
	FootStepSounds(4)=Sound'MoreMiscSounds.loudfootsteps.footstep6h'
	GutsEmitterClass1=Class'AW7FX.TorsoGutsIntestine'
	GutsEmitterClass2=Class'AW7FX.TorsoGutsOrgans'
	HeadClass=class'AW7ZombieHead'
	LimbClass=class'Aw7Limb'
	// ED Zombie Skins
	Skins(0)=Texture'AW7_EDZombies.Misc.XX__142__Fem_SS_Shorts'
	Mesh=SkeletalMesh'AW7Characters.Avg_Zombie'
	ChameleonMeshPkgs(0)="AW7Characters"
	ChameleonSkins(00)="AW7_EDZombies.Bodies_Female.FM__042__Fem_LS_Skirt"
	ChameleonSkins(01)="AW7_EDZombies.Bodies_Female.FM__043__Fem_LS_Skirt"
	ChameleonSkins(02)="AW7_EDZombies.Bodies_Female.FM__044__Fem_LS_Skirt"
	ChameleonSkins(03)="AW7_EDZombies.Bodies_Female.FM__045__Fem_LS_Skirt"
	ChameleonSkins(04)="AW7_EDZombies.Bodies_Female.FM__046__Fem_LS_Skirt"
	ChameleonSkins(05)="AW7_EDZombies.Bodies_Female.FM__047__Fem_SS_Shorts"
	ChameleonSkins(06)="AW7_EDZombies.Bodies_Female.FM__048__Fat_F_SS_Pants"
	ChameleonSkins(07)="AW7_EDZombies.Bodies_Female.FM__049__Fem_LS_Pants"
	ChameleonSkins(08)="AW7_EDZombies.Bodies_Female.FM__050__Fem_LS_Pants"
	ChameleonSkins(09)="AW7_EDZombies.Bodies_Female.FM__051__Fem_LS_Pants"
	ChameleonSkins(10)="AW7_EDZombies.Bodies_Female.FM__052__Fem_LS_Pants"
	ChameleonSkins(11)="AW7_EDZombies.Bodies_Female.FM__053__Fem_LS_Skirt"
	ChameleonSkins(12)="AW7_EDZombies.Bodies_Female.FM__095__Fem_SS_Shorts"
	ChameleonSkins(13)="AW7_EDZombies.Bodies_Female.FW__032__Fem_LS_Pants"
	ChameleonSkins(14)="AW7_EDZombies.Bodies_Female.FW__033__Fem_LS_Pants"
	ChameleonSkins(15)="AW7_EDZombies.Bodies_Female.FW__034__Fem_LS_Pants"
	ChameleonSkins(16)="AW7_EDZombies.Bodies_Female.FW__035__Fat_F_SS_Pants"
	ChameleonSkins(17)="AW7_EDZombies.Bodies_Female.FW__036__Fem_LS_Skirt"
	ChameleonSkins(18)="AW7_EDZombies.Bodies_Female.FW__037__Fem_LS_Skirt"
	ChameleonSkins(19)="AW7_EDZombies.Bodies_Female.FW__038__Fem_LS_Skirt"
	ChameleonSkins(20)="AW7_EDZombies.Bodies_Female.FW__039__Fem_LS_Skirt"
	ChameleonSkins(21)="AW7_EDZombies.Bodies_Female.FW__040__Fem_LS_Skirt"
	ChameleonSkins(22)="AW7_EDZombies.Bodies_Female.FW__041__Fem_LS_Skirt"
	ChameleonSkins(23)="AW7_EDZombies.Bodies_Male.MM__002__Avg_M_Jacket_Pants"
	ChameleonSkins(24)="AW7_EDZombies.Bodies_Male.MM__003__Avg_M_Jacket_Pants"
	ChameleonSkins(25)="AW7_EDZombies.Bodies_Male.MM__004__Avg_M_SS_Pants"
	ChameleonSkins(26)="AW7_EDZombies.Bodies_Male.MM__005__Avg_M_SS_Pants"
	ChameleonSkins(27)="AW7_EDZombies.Bodies_Male.MM__006__Avg_M_SS_Pants"
	ChameleonSkins(28)="AW7_EDZombies.Bodies_Male.MM__007__Avg_M_SS_Pants"
	ChameleonSkins(29)="AW7_EDZombies.Bodies_Male.MM__008__Avg_M_SS_Pants"
	ChameleonSkins(30)="AW7_EDZombies.Bodies_Male.MM__009__Fat_M_SS_Pants"
	ChameleonSkins(31)="AW7_EDZombies.Bodies_Male.MM__010__Avg_Dude"
	ChameleonSkins(32)="AW7_EDZombies.Bodies_Male.MM__011__Avg_Dude"
	ChameleonSkins(33)="AW7_EDZombies.Bodies_Male.MM__012__Avg_M_SS_Pants"
	ChameleonSkins(34)="AW7_EDZombies.Bodies_Male.MM__013__Avg_M_SS_Pants"
	ChameleonSkins(35)="AW7_EDZombies.Bodies_Male.MM__014__Fat_M_SS_Pants"
	ChameleonSkins(36)="AW7_EDZombies.Bodies_Male.MM__015__Fat_M_Jacket_Pants"
	ChameleonSkins(37)="AW7_EDZombies.Bodies_Male.MM__018__Avg_M_Jacket_Pants"
	ChameleonSkins(38)="AW7_EDZombies.Bodies_Male.MM__052__Avg_M_SS_Pants"
	ChameleonSkins(39)="AW7_EDZombies.Bodies_Male.MM__061__Avg_M_SS_Shorts"
	ChameleonSkins(40)="AW7_EDZombies.Bodies_Male.MM__090__Big_M_LS_Pants"
	ChameleonSkins(41)="AW7_EDZombies.Bodies_Male.MM__102__Fat_M_SS_Pants"
	ChameleonSkins(42)="AW7_EDZombies.Bodies_Male.MM__105__Fat_M_SS_Pants"
	ChameleonSkins(43)="AW7_EDZombies.Bodies_Male.MM__107__Fat_M_SS_Pants"
	ChameleonSkins(44)="AW7_EDZombies.Bodies_Male.MW__016__Avg_Dude"
	ChameleonSkins(45)="AW7_EDZombies.Bodies_Male.MW__017__Avg_Dude"
	ChameleonSkins(46)="AW7_EDZombies.Bodies_Male.MW__018__Avg_M_Jacket_Pants"
	ChameleonSkins(47)="AW7_EDZombies.Bodies_Male.MW__019__Avg_M_Jacket_Pants"
	ChameleonSkins(48)="AW7_EDZombies.Bodies_Male.MW__020__Avg_M_Jacket_Pants"
	ChameleonSkins(49)="AW7_EDZombies.Bodies_Male.MW__021__Avg_M_SS_Pants"
	ChameleonSkins(50)="AW7_EDZombies.Bodies_Male.MW__022__Avg_M_SS_Pants"
	ChameleonSkins(51)="AW7_EDZombies.Bodies_Male.MW__023__Avg_M_SS_Pants"
	ChameleonSkins(52)="AW7_EDZombies.Bodies_Male.MW__024__Avg_M_SS_Pants"
	ChameleonSkins(53)="AW7_EDZombies.Bodies_Male.MW__025__Avg_M_SS_Pants"
	ChameleonSkins(54)="AW7_EDZombies.Bodies_Male.MW__026__Avg_M_SS_Pants"
	ChameleonSkins(55)="AW7_EDZombies.Bodies_Male.MW__027__Fat_M_Jacket_Pants"
	ChameleonSkins(56)="AW7_EDZombies.Bodies_Male.MW__028__Big_M_LS_Pants"
	ChameleonSkins(57)="AW7_EDZombies.Bodies_Male.MW__029__Big_M_LS_Pants"
	ChameleonSkins(58)="AW7_EDZombies.Bodies_AW.FW__200__Fem_LS_Pants"
	ChameleonSkins(59)="AW7_EDZombies.Bodies_AW.FW__201__Fem_LS_Skirt"
	ChameleonSkins(60)="AW7_EDZombies.Bodies_AW.FW__208__Fem_LS_Pants"
	ChameleonSkins(61)="AW7_EDZombies.Bodies_AW.MM__206__Avg_Dude"
	ChameleonSkins(62)="AW7_EDZombies.Bodies_AW.MW__202__Avg_M_Jacket_Pants"
	ChameleonSkins(63)="AW7_EDZombies.Bodies_AW.MW__203__Avg_M_Jacket_Pants"
	ChameleonSkins(64)="AW7_EDZombies.Bodies_AW.MW__204__Avg_M_SS_Pants"
	ChameleonSkins(65)="AW7_EDZombies.Bodies_AW.MW__205__Avg_M_SS_Pants"
	ChameleonSkins(66)="AW7_EDZombies.Bodies_AW.MW__207__Avg_M_SS_Pants"
	ChameleonSkins(67)="End"
	ChamelHeadSkins(00)="AW7_EDZombies.Heads_Female.FMA__080__FemSHcropped"
	ChamelHeadSkins(01)="AW7_EDZombies.Heads_Female.FMA__081__FemLH"
	ChamelHeadSkins(02)="AW7_EDZombies.Heads_Female.FMA__082__FemSH"
	ChamelHeadSkins(03)="AW7_EDZombies.Heads_Female.FMA__083__FemSH"
	ChamelHeadSkins(04)="AW7_EDZombies.Heads_Female.FMA__084__FemSH"
	ChamelHeadSkins(05)="AW7_EDZombies.Heads_Female.FMA__085__FemSH"
	ChamelHeadSkins(06)="AW7_EDZombies.Heads_Female.FMA__086__FemSHcropped"
	ChamelHeadSkins(07)="AW7_EDZombies.Heads_Female.FMA__087__FemSHcropped"
	ChamelHeadSkins(08)="AW7_EDZombies.Heads_Female.FMA__088__FemSH"
	ChamelHeadSkins(09)="AW7_EDZombies.Heads_Female.FWA__089__FemSH"
	ChamelHeadSkins(10)="AW7_EDZombies.Heads_Female.FWA__090__FemSH"
	ChamelHeadSkins(11)="AW7_EDZombies.Heads_Female.FWA__091__FemSH"
	ChamelHeadSkins(12)="AW7_EDZombies.Heads_Female.FWA__092__FemLH"
	ChamelHeadSkins(13)="AW7_EDZombies.Heads_Male.MBA__013__AvgBrotha"
	ChamelHeadSkins(14)="AW7_EDZombies.Heads_Male.MBA__014__AvgBrotha"
	ChamelHeadSkins(15)="AW7_EDZombies.Heads_Male.MMA__003__AvgMale"
	ChamelHeadSkins(16)="AW7_EDZombies.Heads_Male.MMA__016__AvgMale"
	ChamelHeadSkins(17)="AW7_EDZombies.Heads_Male.MMA__054__AvgMale"
	ChamelHeadSkins(18)="AW7_EDZombies.Heads_Male.MMA__055__AvgMale"
	ChamelHeadSkins(19)="AW7_EDZombies.Heads_Male.MMA__056__AvgMale"
	ChamelHeadSkins(20)="AW7_EDZombies.Heads_Male.MMA__057__AvgMale"
	ChamelHeadSkins(21)="AW7_EDZombies.Heads_Male.MMA__058__AvgMale"
	ChamelHeadSkins(22)="AW7_EDZombies.Heads_Male.MMA__059__AvgMale"
	ChamelHeadSkins(23)="AW7_EDZombies.Heads_Male.MMA__060__AvgMale"
	ChamelHeadSkins(24)="AW7_EDZombies.Heads_Male.MMA__061__AvgMale"
	ChamelHeadSkins(25)="AW7_EDZombies.Heads_Male.MMA__062__AvgMale"
	ChamelHeadSkins(26)="AW7_EDZombies.Heads_Male.MMA__063__AvgMale"
	ChamelHeadSkins(27)="AW7_EDZombies.Heads_Male.MMA__064__AvgMale"
	ChamelHeadSkins(28)="AW7_EDZombies.Heads_Male.MMA__065__AvgMale"
	ChamelHeadSkins(29)="AW7_EDZombies.Heads_Male.MWA__066__AvgMale"
	ChamelHeadSkins(30)="AW7_EDZombies.Heads_Male.MWA__067__AvgMale"
	ChamelHeadSkins(31)="AW7_EDZombies.Heads_Male.MWA__068__AvgMale"
	ChamelHeadSkins(32)="AW7_EDZombies.Heads_Male.MWA__069__AvgMale"
	ChamelHeadSkins(33)="AW7_EDZombies.Heads_Male.MWA__070__AvgMale"
	ChamelHeadSkins(34)="AW7_EDZombies.Heads_Male.MWA__071__AvgMale"
	ChamelHeadSkins(35)="AW7_EDZombies.Heads_Male.MWA__072__AvgMale"
	ChamelHeadSkins(36)="AW7_EDZombies.Heads_Male.MWA__073__AvgMale"
	ChamelHeadSkins(37)="AW7_EDZombies.Heads_Male.MWA__074__AvgMale"
	ChamelHeadSkins(38)="AW7_EDZombies.Heads_Male.MWA__075__AvgMale"
	ChamelHeadSkins(39)="AW7_EDZombies.Heads_Male.MWA__076__AvgMale"
	ChamelHeadSkins(40)="AW7_EDZombies.Heads_Male.MWA__077__AvgMale"
	ChamelHeadSkins(41)="AW7_EDZombies.Heads_Male.MWA__078__FatMale"
	ChamelHeadSkins(42)="AW7_EDZombies.Heads_Male.MWA__079__FatMale"
	ChamelHeadSkins(43)="AW7_EDZombies.Heads_AW.FWA__200__FemSH"
	ChamelHeadSkins(44)="AW7_EDZombies.Heads_AW.FWA__201__FemSH"
	ChamelHeadSkins(45)="AW7_EDZombies.Heads_AW.FWA__202__FemSH"
	ChamelHeadSkins(46)="AW7_EDZombies.Heads_AW.FWA__203__FemSH"
	ChamelHeadSkins(47)="AW7_EDZombies.Heads_AW.FWA__204__FemSHcropped"
	ChamelHeadSkins(48)="AW7_EDZombies.Heads_AW.MMA__213__AvgMale"
	ChamelHeadSkins(49)="AW7_EDZombies.Heads_AW.MMA__215__AvgMale"
	ChamelHeadSkins(50)="AW7_EDZombies.Heads_AW.MWA__205__AvgMale"
	ChamelHeadSkins(51)="AW7_EDZombies.Heads_AW.MWA__206__AvgMale"
	ChamelHeadSkins(52)="AW7_EDZombies.Heads_AW.MWA__207__AvgMale"
	ChamelHeadSkins(53)="AW7_EDZombies.Heads_AW.MWA__208__AvgMale"
	ChamelHeadSkins(54)="AW7_EDZombies.Heads_AW.MWA__209__AvgMale"
	ChamelHeadSkins(55)="AW7_EDZombies.Heads_AW.MWA__210__AvgMale"
	ChamelHeadSkins(56)="AW7_EDZombies.Heads_AW.MWA__211__AvgMale"
	ChamelHeadSkins(57)="AW7_EDZombies.Heads_AW.MWA__212__AvgMale"
	ChamelHeadSkins(58)="AW7_EDZombies.Heads_AW.MWA__214__AvgMale"
	ChamelHeadSkins(59)="AW7_EDZombies.Heads_AW.MWA__216__AvgMale"
	ChamelHeadSkins(60)="AW7_EDZombies.Heads_AW.MWA__217__AvgMale"
	ChamelHeadSkins(61)="End"
	ChamelHeadMeshPkgs(0)="heads"
}
