///////////////////////////////////////////////////////////////////////////////
// AWZombieController
// Copyright 2004 RWS, Inc.  All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////
class AW7ZombieController extends AWZombieController;

///////////////////////////////////////////////////////////////////////////////
// Consts 'n' stuff
///////////////////////////////////////////////////////////////////////////////
const KMASS_DAMAGEMULT			= 5.00;

const MAX_ROT_DIFF = 1024;

///////////////////////////////////////////////////////////////////////////////
// Bystanders have view pitching now
///////////////////////////////////////////////////////////////////////////////
simulated event Tick(float Delta)
{
	local rotator ORot;
	local vector VDiff;
	local float PDiff;

	Super.Tick(Delta);

	if (Focus != None && !Focus.bStatic)
	{
		VDiff = Focus.Location - Pawn.Location;
		ORot = Rotator(VDiff);
		// Only look up/down at focus if we're looking directly at them, otherwise it looks stupid
		if (CanSeePoint(Pawn, Focus.Location)
			&& !IsInState('ThrownThroughAir')
			&& !IsInState('DoPuking')
			&& !IsInState('BeingShocked')
			&& !IsInState('RestFromAttacker')
			&& !IsInState('DanceHere')
			&& !IsInState('PlayArcadeGame')
			&& !IsInState('AttackedByChompy')
			&& !IsInState('WipeFace')
			&& !IsInState('PrepDeathCrawl')
			&& !IsInState('RunOnFire')
			&& !IsInState('DeathCrawlFromAttacker')
			&& !IsInState('CowerInABall')
			&& !IsInState('ImOnFire')
			)
			Pawn.ViewPitch = Clamp(ORot.Pitch / 256, 0, 255);
		else
			Pawn.ViewPitch = 0;
//		log(self@"focus is"@Focus@"VDiff"@VDiff@"ORot"@ORot@"Rotation"@Pawn.Rotation);
	}
	else
		Pawn.ViewPitch = 0;
}

///////////////////////////////////////////////////////////////////////////////
// Some karma actors hurt us
///////////////////////////////////////////////////////////////////////////////
function CheckForKarmaDamage(Actor Other)
{
	local int DamageTaken;

	// See if it's a kactor. Ignore carexplodables, they hurt us themselves
	if (ClassIsChildOf(Other.Class,class'KActor') && Other.Instigator != None && !Other.IsA('CarExplodable'))
	{
		// Below a certain mass, get pissed and flip off the player
		if (Other.KGetMass() <= 2)
			InterestIsAnnoyingUs(Other.Instigator, False);
		else // Take damage; possibly retaliate
		{
			DamageTaken = Other.KGetMass() * KMASS_DAMAGEMULT;
			Pawn.TakeDamage(DamageTaken, Other.Instigator, Location, Other.Velocity, class'SmashDamage');
			DamageAttitudeTo(Other.Instigator, DamageTaken);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Some karma actors hurt us
///////////////////////////////////////////////////////////////////////////////
event Bump(actor Other)
{
	CheckForKarmaDamage(Other);
	Super.Bump(Other);
}

///////////////////////////////////////////////////////////////////////////////
// Some karma actors hurt us
///////////////////////////////////////////////////////////////////////////////
event Touch(actor Other)
{
	CheckForKarmaDamage(Other);
	Super.Touch(Other);
}

///////////////////////////////////////////////////////////////////////////////
// Ignore intruders in stasis (this is a problem for some reason)
///////////////////////////////////////////////////////////////////////////////
state StasisState
{
	function CheckForIntruder(FPSPawn LookAtMe, out byte StateChange);
	function HandleIntruder(FPSPawn LookAtMe);
	event Trigger( Actor Other, Pawn EventInstigator );
	function bool SeeWeaponDrop(P2WeaponPickup grabme);
	function WeaponDropped(P2WeaponPickup grabme);
}

///////////////////////////////////////////////////////////////////////////////
// Ignore intruders in stasis (this is a problem for some reason)
///////////////////////////////////////////////////////////////////////////////
state SliderStasis
{
	function CheckForIntruder(FPSPawn LookAtMe, out byte StateChange);
	function HandleIntruder(FPSPawn LookAtMe);
	event Trigger( Actor Other, Pawn EventInstigator );
	function bool SeeWeaponDrop(P2WeaponPickup grabme);
	function WeaponDropped(P2WeaponPickup grabme);
}

defaultproperties
{
}
