///////////////////////////////////////////////////////////////////////////////
// AW7MenuCheatsBase
// By: Kamek (kamek@postalleague.com)
// Base: AWMenuCheats
// For: AW7
//
// Base class for AW7 cheat menus.
///////////////////////////////////////////////////////////////////////////////
class AW7MenuCheatsBase extends ShellMenuCW;

///////////////////////////////////////////////////////////////////////////////
// Const
///////////////////////////////////////////////////////////////////////////////
const CHEATS_MAX = 15;

///////////////////////////////////////////////////////////////////////////////
// Public vars
///////////////////////////////////////////////////////////////////////////////
var() string CheatsTitleText;					// Menu title
var() string CheatsExplainText;					// How to enable cheats
var() string CheatsExplainText2;
var() string CheatsText[CHEATS_MAX];			// Name of cheat
var() string CheatsHelp[CHEATS_MAX];			// Explanation of cheat
var() class<ShellMenuCW> MenuMoreCheatsClass;	// Class of next cheat menu

var() const int Blank_MP1;
var() const int Blank_MP1a;
var() const int Blank_MP2;
var() const int Blank_MP3;
var() const int Blank_Metool;
var() const int Blank_Busta;
var() const int Blank_Fathom;
var() const int Blank_Matrix;
var() const int Blank_Matrix2;
var() const int Blank_Matrix3;
var() const int Blank_AllCheatsA;
var() const int Blank_AllCheatsB;
var() const int Blank_AllCheatsC;
var() const int Blank_AllCheatsD;
var() const int Blank_AllCheatsE;

///////////////////////////////////////////////////////////////////////////////
// Internal
///////////////////////////////////////////////////////////////////////////////
var ShellMenuChoice	CheatsExplainChoice;
var ShellMenuChoice	CheatsExplainChoice2;
var ShellMenuChoice	NextChoice;
var ShellMenuChoice CheatsChoice[CHEATS_MAX];
var string BlankText;

///////////////////////////////////////////////////////////////////////////////
// Create menu contents
///////////////////////////////////////////////////////////////////////////////
function CreateMenuContents()
	{
	local int i;
	local AW7GameInfo useinfo;

	Super.CreateMenuContents();

	ItemFont	= F_FancyM;
	ItemAlign = TA_Center;
	AddTitle(CheatsTitleText, TitleFont, TitleAlign);
	CheatsExplainChoice=AddChoice(CheatsExplainText, "", ItemFont, TitleAlign);
	CheatsExplainChoice.bActive=false;
	CheatsExplainChoice2=AddChoice(CheatsExplainText2, "", ItemFont, TitleAlign);
	CheatsExplainChoice2.bActive=false;

	// Blank out cheats that haven't been earned yet
	useinfo = AW7GameInfo(GetGameSingle());
	if(Blank_MP1 != -1 && !useinfo.DoMP1())
	{

		CheatsText[Blank_MP1] = BlankText;
		CheatsHelp[Blank_MP1] = BlankText;
	}
	if(Blank_MP1a != -1 && !useinfo.DoMP1())
	{
		CheatsText[Blank_MP1a] = BlankText;
		CheatsHelp[Blank_MP1a] = BlankText;
	}
	if(Blank_MP2 != -1 && !useinfo.DoMP2())
	{
		CheatsText[Blank_MP2] = BlankText;
		CheatsHelp[Blank_MP2] = BlankText;
	}
	if(Blank_MP3 != -1 && !useinfo.DoMP3())
	{
		CheatsText[Blank_MP3] = BlankText;
		CheatsHelp[Blank_MP3] = BlankText;
	}
	if (Blank_Metool != -1 && !UseInfo.MetoolEnabled())
	{
		CheatsText[Blank_Metool] = BlankText;
		CheatsHelp[Blank_Metool] = BlankText;
	}
	if (Blank_Busta != -1 && !UseInfo.BustaEnabled())
	{
		CheatsText[Blank_Busta] = BlankText;
		CheatsHelp[Blank_Busta] = BlankText;
	}
	if (Blank_Fathom != -1 && !UseInfo.FathomEnabled())
	{
		CheatsText[Blank_Fathom] = BlankText;
		CheatsHelp[Blank_Fathom] = BlankText;
	}
	if (Blank_Matrix != -1 && !UseInfo.MatrixEnabled())
	{
		CheatsText[Blank_Matrix] = BlankText;
		CheatsHelp[Blank_Matrix] = BlankText;
	}
	if (Blank_Matrix2 != -1 && !UseInfo.MatrixEnabled())
	{
		CheatsText[Blank_Matrix2] = BlankText;
		CheatsHelp[Blank_Matrix2] = BlankText;
	}
	if (Blank_AllCheatsA != -1 && !UseInfo.AllEnabled())
	{
		CheatsText[Blank_AllCheatsA] = BlankText;
		CheatsHelp[Blank_AllCheatsA] = BlankText;
	}
	if (Blank_AllCheatsB != -1 && !UseInfo.AllEnabled())
	{
		CheatsText[Blank_AllCheatsB] = BlankText;
		CheatsHelp[Blank_AllCheatsB] = BlankText;
	}
	if (Blank_AllCheatsC != -1 && !UseInfo.AllEnabled())
	{
		CheatsText[Blank_AllCheatsC] = BlankText;
		CheatsHelp[Blank_AllCheatsC] = BlankText;
	}
	if (Blank_AllCheatsD != -1 && !UseInfo.AllEnabled())
	{
		CheatsText[Blank_AllCheatsD] = BlankText;
		CheatsHelp[Blank_AllCheatsD] = BlankText;
	}
	if (Blank_AllCheatsE != -1 && !UseInfo.AllEnabled())
	{
		CheatsText[Blank_AllCheatsE] = BlankText;
		CheatsHelp[Blank_AllCheatsE] = BlankText;
	}

	for(i=0; i<CHEATS_MAX; i++)
	{
		if (CheatsText[i] != "")
		{
			CheatsChoice[i]	= AddChoice(CheatsText[i], CheatsHelp[i], ItemFont, ItemAlign);
			if (CheatsText[i] == BlankText)
				CheatsChoice[i].bActive = false;
		}
	}

	if (MenuMoreCheatsClass != None)
		NextChoice			= AddChoice(NextText, "", ItemFont, TitleAlign);

	BackChoice			= AddChoice(BackText, "", ItemFont, TitleAlign, true);

	// If you start up this menu, make sure to enable cheats for you
	GetPlayerOwner().ConsoleCommand("ForceCheatingCheater");
	}

///////////////////////////////////////////////////////////////////////////////
// Handle notifications from controls
///////////////////////////////////////////////////////////////////////////////
function Notify(UWindowDialogControl C, byte E)
	{
	local int i;

	Super.Notify(C, E);
	
	if (E == DE_Click)
	{
		for (i=0; i<CHEATS_MAX; i++)
			if (C == CheatsChoice[i])
			{
				GetPlayerOwner().ConsoleCommand(CheatsText[i]);
				break;
			}
	}

	switch(E)
		{
		case DE_Click:
			if (C != None)
				switch (C)
					{
					case NextChoice:
						GotoMenu(MenuMoreCheatsClass);
						break;
					case BackChoice:
						GoBack();
						break;
					}
			break;
		}
	}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
	MenuWidth=500.000000
    ItemHeight=20.000000
	BlankText="? ? ? ? ?"
	CheatsTitleText="Cheats"
	CheatsExplainText="Click below to get the cheat,"
	CheatsExplainText2="then resume the game to use it."
	Blank_MP1=-1
	Blank_MP1a=-1
	Blank_MP2=-1
	Blank_MP3=-1
	Blank_Metool=-1
	Blank_Busta=-1
	Blank_Fathom=-1
	Blank_Matrix=-1
	Blank_Matrix2=-1
	Blank_Matrix3=-1
	Blank_AllCheatsA=-1
	Blank_AllCheatsB=-1
	Blank_AllCheatsC=-1
	Blank_AllCheatsD=-1
	Blank_AllCheatsE=-1
}
