///////////////////////////////////////////////////////////////////////////////
// AWMenuStart.uc
// Copyright 2003 Running With Scissors, Inc.  All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////
class AW7MenuStart extends AWMenuStart;

var string StartGameAW7;
var string StartGameWeekend;
var string StartGameMF;
//var string StartGameOrig;
//var string StartGameP2;

var UWindowCheckBox EnhancedCheckbox;
var ShellMenuChoice StartAW7;
var ShellMenuChoice StartMF;
var ShellMenuChoice StartWeekend;
//var ShellMenuChoice StartOrig;
//var ShellMenuChoice StartP2;

var localized string EnhancedText;
var localized string StartAW7Text;
var localized string StartMFText;
var localized string StartWeekendText;
//var localized string StartOrigText;
//var localized string StartP2Text;

var localized string EnhancedHelp;
var localized string StartAW7Help;
var localized string StartMFHelp;
var localized string StartWeekendHelp;
//var localized string StartOrigHelp;
//var localized string StartP2Help;

///////////////////////////////////////////////////////////////////////////////
// Create menu contents
///////////////////////////////////////////////////////////////////////////////
function CreateMenuContents()
	{
	Super(ShellMenuCW).CreateMenuContents();

	AddTitle(TitleText, F_FancyXL, TA_Left);

	ItemFont = F_FancyL;
	DifficultyCombo = AddComboBox(DifficultyText, DifficultyHelp, ItemFont);
	DifficultyCombo.List.MaxVisible = ArrayCount(AW7GameInfo(GetPlayerOwner().Level.Game).DifficultyNames);

	if (GetGameSingle().FinallyOver())
	{
		EnhancedCheckbox = AddCheckbox(EnhancedText, EnhancedHelp, ItemFont);
		EnhancedCheckbox.SetValue(False);
	}

	StartAW7 =		AddChoice(StartAW7Text,		StartAW7Help,		ItemFont,	TA_Left);
	StartMF =		AddChoice(StartMFText,		StartMFHelp,		ItemFont,	TA_Left);
	StartWeekend =	AddChoice(StartWeekendText,	StartWeekendHelp,	ItemFont,	TA_Left);
//	StartOrig =		AddChoice(StartOrigText,	StartOrigHelp,		ItemFont,	TA_Left);
//	StartP2 =		AddChoice(StartP2Text,		StartP2Help,		ItemFont,	TA_Left);

	BackChoice =	AddChoice(BackText,   	"",				ItemFont,	TA_Left, true);

	LoadValues();
	}

///////////////////////////////////////////////////////////////////////////////
// Load all values from ini files
///////////////////////////////////////////////////////////////////////////////
function LoadValues()
{
	local float val;
	local bool flag;
	local String detail;
	local int i;
	local AW7GameInfo psg;

	psg = AW7GameInfo(GetPlayerOwner().Level.Game);

	// Controls will generate Notify() events when their values are updated, so we
	// use this flag to block the events from actually doing anything.  In other
	// words, we're only setting the initial values of the controls and we don't
	// want that to count as a change.
	bUpdate = False;

	if (DifficultyCombo != none)
		{
		DifficultyCombo.Clear();

		for(i=0; i<ArrayCount(psg.DifficultyNames); i++)
			DifficultyCombo.AddItem(psg.DifficultyNames[i]);

		val = int(GetPlayerOwner().ConsoleCommand("get"@c_strDifficultyPath));

		if(val > 0)
			DifficultyCombo.SetValue(psg.DifficultyNames[int(val)]);

		// Seems too wide on the text side.
		DifficultyCombo.EditBoxWidth = DifficultyCombo.WinWidth * 0.50;
		}
	bUpdate = True;
}

///////////////////////////////////////////////////////////////////////////////
// Update values when controls have changed.
// - if bUpdate is false then don't update the real value
// - if bAsk is false then skip any user confirmations
///////////////////////////////////////////////////////////////////////////////
function DiffChanged(bool bUpdate)
	{
	local string diffname;
	local int val;
	local AW7GameInfo psg;

	if (bUpdate)
		{

		psg = AW7GameInfo(GetPlayerOwner().Level.Game);

		diffname = DifficultyCombo.GetValue();
		//log(self$" DiffChanged diffname after change "$diffname);
		if(diffname == psg.DifficultyNames[0])
			val = 0;
		else if(diffname == psg.DifficultyNames[1])
			val = 1;
		else if(diffname == psg.DifficultyNames[2])
			val = 2;
		else if(diffname == psg.DifficultyNames[3])
			val = 3;
		else if(diffname == psg.DifficultyNames[4])
			val = 4;
		else if(diffname == psg.DifficultyNames[5])
			val = 5;
		else if(diffname == psg.DifficultyNames[6])
			val = 6;
		else if(diffname == psg.DifficultyNames[7])
			val = 7;
		else if(diffname == psg.DifficultyNames[8])
			val = 8;
		else if(diffname == psg.DifficultyNames[9])
			val = 9;
		else if(diffname == psg.DifficultyNames[10])
			val = 10;
		else if(diffname == psg.DifficultyNames[11])
			val = 11;
		else if(diffname == psg.DifficultyNames[12])
			val = 12;
		else if(diffname == psg.DifficultyNames[13])
			val = 13;
		//log(self$" DiffChanged diff value "$val);
		// set diff
		GetPlayerOwner().ConsoleCommand("set"@c_strDifficultyPath@val);
		psg.GameDifficulty = val;
		// Update the gamestate here, also, if we have one
		if(psg.TheGameState != None)
			psg.TheGameState.GameDifficulty = val;
		}
	}

///////////////////////////////////////////////////////////////////////////////
// Allows for use of enhanced mode
///////////////////////////////////////////////////////////////////////////////
function StartGame2(bool bEnhanced)
{
	local P2Player p2p;
	local P2GameInfoSingle usegame;

	usegame = GetGameSingle();
	p2p = usegame.GetPlayer();
	P2RootWindow(Root).StartingGame();

	// Stop any active SceneManager so player will have a pawn
	usegame.StopSceneManagers();

	usegame.PrepIniStartVals();
	usegame.TheGameState.bEGameStart = bEnhanced;

	// Get the difficulty ready for this game state.
	usegame.SetupDifficultyOnce();

	// Get rid of any things in his inventory before a new game starts
	P2Pawn(p2p.pawn).DestroyAllInventory();

	// Game doesn't actually start until player is sent to first day,
	// which *should* happen at the end of the intro sequence.
	usegame.TheGameState.bChangeDayPostTravel = true;
	usegame.TheGameState.NextDay = 0;
	// Actually start the game with the first level
	usegame.SendPlayerTo(p2p, StartGameURL);
}

///////////////////////////////////////////////////////////////////////////////
// Handle notifications from controls
///////////////////////////////////////////////////////////////////////////////
function Notify(UWindowDialogControl C, byte E)
	{
	local int val;

	Super(ShellMenuCW).Notify(C, E);
	switch(E)
		{
		case DE_Change:
			switch (C)
			{
				case DifficultyCombo:
					DiffChanged(bUpdate);
					break;
			}
			break;
		case DE_Click:
			switch (C)
			{
				case BackChoice:
					GoBack();
					break;
				case StartAW7:
					if (!AW7GameInfo(GetPlayerOwner().Level.Game).InNightmareMode())
					{
						StartGameURL = StartGameAW7;
						StartGame2(EnhancedCheckbox.GetValue());
						ShellRootWindow(Root).bLaunchedMultiplayer = false;
					}
					else
					{
						AW7RootWindow(Root).bEnhancedMode = EnhancedCheckbox.GetValue();
						AW7RootWindow(Root).StartGameURL = StartGameAW7;
						GotoMenu(class'AW7MenuNightmare');
					}
					break;
				case StartMF:
					if (!AW7GameInfo(GetPlayerOwner().Level.Game).InNightmareMode())
					{
						StartGameURL = StartGameMF;
						StartGame2(EnhancedCheckbox.GetValue());
						ShellRootWindow(Root).bLaunchedMultiplayer = false;
					}
					else
					{
						AW7RootWindow(Root).bEnhancedMode = EnhancedCheckbox.GetValue();
						AW7RootWindow(Root).StartGameURL = StartGameMF;
						GotoMenu(class'AW7MenuNightmare');
					}
					break;
				case StartWeekend:
					if (!AW7GameInfo(GetPlayerOwner().Level.Game).InNightmareMode())
					{
						StartGameURL = StartGameWeekend;
						GetGameSingle().TheGameState.CurrentDay = 6; // fool the loading screen into thinking it's Saturday
						StartGame2(EnhancedCheckbox.GetValue());
						ShellRootWindow(Root).bLaunchedMultiplayer = false;
					}
					else
					{
						AW7RootWindow(Root).bEnhancedMode = EnhancedCheckbox.GetValue();
						AW7RootWindow(Root).StartGameURL = StartGameWeekend;
						GotoMenu(class'AW7MenuNightmare');
					}
					break;
/*
				case StartOrig:
					StartGameURL = StartGameOrig;
					GetGameSingle().TheGameState.CurrentDay = 6; // fool the loading screen into thinking it's Saturday
					StartGame2(EnhancedCheckbox.GetValue());
					ShellRootWindow(Root).bLaunchedMultiplayer = false;
					break;
				case StartP2:
					StartGameURL = StartGameP2;
					StartGame2(EnhancedCheckbox.GetValue());
					ShellRootWindow(Root).bLaunchedMultiplayer = false;
					break;
*/
			}
			break;
		}
}

defaultproperties
{
	MenuWidth=375.000000
	MenuHeight=400.000000
	TitleSpacingY=30.000000
	ItemSpacingY = 10

	StartGameAW7="AW7_intro.fuk?Game=AW7Game.AW7GameInfo"
	StartGameWeekend="AW7_WeekendIntro.fuk?Game=AW7Game.AW7GameWeekend"
//	StartGameOrig="MovieIntro.fuk?Game=AW7Game.AW7AWGameSingle"
	StartGameMF="AW7_intro.fuk?Game=AW7Game.AW7GameMF"
//	StartGameP2="Intro.fuk?Game=AW7Game.AW7P2GameSingle"
	StartAW7Help="Play all seven days"
	StartMFHelp="Play Monday through Friday only"
	StartWeekendHelp="Play Weekend only"
//	StartOrigHelp="Play original AW game (no AWP)"
//	StartP2Help="Play the original Postal 2 (no AW or AWP)"
	StartAW7Text="AW7 Start"
	StartMFText="M-F Start"
	StartWeekendText="Weekend Start"
//	StartOrigText="Original AW Start"
//	StartP2Text="Original Postal 2 Start"
	EnhancedText="Enhanced Game"
	EnhancedHelp="Play the Enhanced Game"
}
